{Ŀ}
{ MODULE:         DevAdLib                                                 }
{                                                                          }
{ DESCRIPTION:    Device driver for the Adlib sound card and compatibles.  }
{                 This works with Sound Blasters and with Covox SM II.     }
{                 It uses timer polling.                                   }
{                                                                          }
{ DEVELOPER:      Carlos daSilva                                           }
{}

UNIT DevAdLib;

INTERFACE

{Ŀ}
{ Device ID string                                                         }
{}
CONST AdlibDevID = 'AdLib';

IMPLEMENTATION
USES SoundDevices, StrConst;

{Ŀ}
{ AdLib routines                                                           }
{}
PROCEDURE AdLibLoop; ASSEMBLER;
ASM
@@lp:
  IN      AL,DX
  LOOP    @@lp
END;

PROCEDURE SendAdlibReg(Reg, Value : Byte); ASSEMBLER;
ASM
  MOV     DX,$388
  MOV     AL,Reg
  OUT     DX,AL

  MOV     CX,6
  CALL    AdLibLoop

  INC     DX
  MOV     AL,Value
  OUT     DX,AL

  MOV     CX,35
  CALL    AdLibLoop
END;

PROCEDURE ResetAdlib;
VAR I : Word;
BEGIN
  For I := 1 to 255 do
    SendAdlibReg(I, 0);
END;

{Ŀ}
{ Assembler IRQ                                                            }
{}
{$L DEVADLIB}

PROCEDURE AdLibIntHandler; FAR; EXTERNAL;

{Ŀ}
{ Name function                                                            }
{}
FUNCTION AdLibName : tDevName; FAR;
BEGIN
  AdLibName := GetString(StrDevAdLibName);
END;

{Ŀ}
{ Autodetect routine (cannot be detected)                                  }
{}
FUNCTION DevDetect : Boolean; FAR;
BEGIN
  DevDetect := True;
END;

{Ŀ}
{ Device initialization routine                                            }
{}
PROCEDURE DevInit(Hz : Word); FAR;
BEGIN
  ResetAdlib;             { Turn off all channels, and stuff. }
                          { Start in a known state.           }
  SendAdlibReg($01, $20);
  SendAdlibReg($C0, $01);
  SendAdlibReg($23, $27); { All this selects a sustained note, with a }
  SendAdlibReg($43, $00); { highest pitch and with a fully rectified  }
  SendAdlibReg($63, $FF); { wave form.  Empirically calculated (trial }
  SendAdlibReg($83, $0F); { (trial and error.)  If you can find a     }
  SendAdlibReg($E3, $02); { better configuration, please tell me.     }
  SendAdlibReg($A0, $0C);
  SendAdlibReg($B0, $3F);

  CalcTimerData(Hz);      { Then, standard polling init. }
  InitTimer;
END;

{Ŀ}
{ Device deinitialization routine                                          }
{}
PROCEDURE DevEnd; FAR;
BEGIN
  ResetAdlib;
END;

{Ŀ}
{ Accessory routines                                                       }
{}
PROCEDURE DevChgHz(Hz : Word); FAR;
BEGIN
  CalcTimerData(Hz);
  InitTimer;
END;

PROCEDURE DevPoll; FAR;
BEGIN
END;

{Ŀ}
{ Device record                                                            }
{}
CONST AdLibData : tSoundDevice =
      (DevID : AdLibDevID;
       DMA   : FALSE);

{Ŀ}
{ Initialization code                                                      }
{}
BEGIN
  WITH AdLibData DO
  BEGIN
    Name            := AdLibName;
    AutoDetect      := DevDetect;
    InitRut         := DevInit;
    ChgHzProc       := DevChgHz;
    GetRealFreqProc := GetRealFreq;
    TimerHandler    := AdLibIntHandler;
    PollRut         := DevPoll;
    EndRut          := DevEnd;
  END;

  InitDevice(@AdLibData);
END.
