{Ŀ}
{ MODULE:         DevDac                                                   }
{                                                                          }
{ DESCRIPTION:    Device driver for all DAC-type sound cards.  Uses timer  }
{                 polling                                                  }
{                                                                          }
{ DEVELOPER:      Carlos daSilva                                           }
{}
UNIT DevDAC;

INTERFACE

{Ŀ}
{ Device ID strings                                                        }
{}
CONST DACDevID      = 'DAC-Mono';
      DAC2DevID     = 'DAC-Stereo';
      MixDAC2DevID  = 'Mix-DAC-Stereo';
      Mix2DAC2DevID = 'Mix2-DAC-Stereo';

{Ŀ}
{ Configuration                                                            }
{}
CONST DacPort  : WORD = $378;
      LDacPort : WORD = $378;
      RDacPort : WORD = $378;

IMPLEMENTATION
USES SoundDevices, StrConst;

{Ŀ}
{ Assembler IRQs                                                           }
{}
{$L DEVDAC}

PROCEDURE DACIntHandler;      FAR; EXTERNAL;
PROCEDURE DAC2IntHandler;     FAR; EXTERNAL;
PROCEDURE MixDAC2IntHandler;  FAR; EXTERNAL;
PROCEDURE Mix2DAC2IntHandler; FAR; EXTERNAL;

{Ŀ}
{ Name functions                                                           }
{}
FUNCTION DACName : tDevName; FAR;
BEGIN
  DACName := GetString(StrDevDACName);
END;

FUNCTION DAC2Name : tDevName; FAR;
BEGIN
  DAC2Name := GetString(StrDevDAC2Name);
END;

FUNCTION MixDAC2Name : tDevName; FAR;
BEGIN
  MixDAC2Name := GetString(StrDevMixDAC2Name);
END;

FUNCTION Mix2DAC2Name : tDevName; FAR;
BEGIN
  Mix2DAC2Name := GetString(StrDevMix2DAC2Name);
END;

{Ŀ}
{ Autodetect routine (cannot be detected)                                  }
{}
FUNCTION DACDetect : Boolean; FAR;
BEGIN
  DACDetect := True;
END;

{Ŀ}
{ Device initialization routine                                            }
{}
PROCEDURE Init(Hz : Word); FAR;
BEGIN
  CalcTimerData(Hz);    { Standard polling initialisation. }
  InitTimer;
END;

{Ŀ}
{ Device deinitialization routine                                          }
{}
PROCEDURE DevEnd; FAR; ASSEMBLER;
ASM
END;

{Ŀ}
{ Accessory routines                                                       }
{}
PROCEDURE DevChgHz(Hz : Word); FAR;
BEGIN
  CalcTimerData(Hz);
  InitTimer;
END;

PROCEDURE DevPoll; FAR;
BEGIN
END;

{Ŀ}
{ Device records                                                           }
{}
CONST DACData : tSoundDevice =
      (DevID      : DACDevID;
       DMA        : False);

      DAC2Data : tSoundDevice =
      (DevID      : DAC2DevID;
       DMA        : False);

      MixDAC2Data : tSoundDevice =
      (DevID      : MixDAC2DevID;
       DMA        : False);

      Mix2DAC2Data : tSoundDevice =
      (DevID      : Mix2DAC2DevID;
       DMA        : False);

{Ŀ}
{ Initialization code                                                      }
{}
BEGIN
  WITH DACData DO
  BEGIN
    Name            := DACName;
    AutoDetect      := DACDetect;
    InitRut         := Init;
    ChgHzProc       := DevChgHz;
    GetRealFreqProc := GetRealFreq;
    TimerHandler    := DACIntHandler;
    PollRut         := DevPoll;
    EndRut          := DevEnd;
  END;

  WITH DAC2Data DO
  BEGIN
    Name            := DAC2Name;
    AutoDetect      := DACDetect;
    InitRut         := Init;
    ChgHzProc       := DevChgHz;
    GetRealFreqProc := GetRealFreq;
    TimerHandler    := DAC2IntHandler;
    PollRut         := DevPoll;
    EndRut          := DevEnd;
  END;

  WITH MixDAC2Data DO
  BEGIN
    Name            := MixDAC2Name;
    AutoDetect      := DACDetect;
    InitRut         := Init;
    ChgHzProc       := DevChgHz;
    GetRealFreqProc := GetRealFreq;
    TimerHandler    := MixDAC2IntHandler;
    PollRut         := DevPoll;
    EndRut          := DevEnd;
  END;

  WITH Mix2DAC2Data DO
  BEGIN
    Name            := Mix2DAC2Name;
    AutoDetect      := DACDetect;
    InitRut         := Init;
    ChgHzProc       := DevChgHz;
    GetRealFreqProc := GetRealFreq;
    TimerHandler    := Mix2DAC2IntHandler;
    PollRut         := DevPoll;
    EndRut          := DevEnd;
  END;

  InitDevice(@Mix2DAC2Data);
  InitDevice(@MixDAC2Data);
  InitDevice(@DAC2Data);
  InitDevice(@DACData);
END.
