{Ŀ}
{ MODULE:         DevSpkr                                                  }
{                                                                          }
{ DESCRIPTION:    Device driver for the PC Internal Speaker.  It uses      }
{                 timer polling.                                           }
{                                                                          }
{ DEVELOPER:      Carlos daSilva                                           }
{}
UNIT DevSpkr;

INTERFACE

{Ŀ}
{ Device ID string                                                         }
{}
CONST SpkrDevID = 'Speaker';

IMPLEMENTATION
USES SoundDevices, StrConst;
{Ŀ}
{ Device data - Default conversion table for 16khz                         }
{}
CONST SpkrBaseTabla : ARRAY[0..255] of Byte =
      ($40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$3F,$3F,$3F,$3F,$3F,$3F,
       $3F,$3F,$3F,$3F,$3F,$3F,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,$3E,
       $3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3D,$3C,$3C,$3C,$3C,$3C,$3C,$3C,
       $3C,$3C,$3C,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3B,$3A,$3A,$3A,
       $3A,$3A,$3A,$3A,$3A,$3A,$3A,$39,$39,$39,$39,$39,$39,$39,$39,$39,
       $39,$38,$38,$38,$38,$38,$38,$38,$38,$37,$37,$37,$37,$37,$36,$36,
       $36,$36,$35,$35,$35,$35,$34,$34,$34,$33,$33,$32,$32,$31,$31,$30,
       $30,$2F,$2E,$2D,$2C,$2B,$2A,$29,$28,$27,$26,$25,$24,$23,$22,$21,
       $20,$1F,$1E,$1D,$1C,$1B,$1A,$19,$18,$17,$16,$15,$14,$13,$12,$11,
       $11,$10,$10,$0F,$0F,$0E,$0E,$0D,$0D,$0D,$0C,$0C,$0C,$0C,$0B,$0B,
       $0B,$0B,$0A,$0A,$0A,$0A,$0A,$09,$09,$09,$09,$09,$09,$09,$09,$09,
       $08,$08,$08,$08,$08,$08,$08,$08,$08,$08,$08,$08,$07,$07,$07,$07,
       $07,$07,$07,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$05,$05,
       $05,$05,$05,$05,$05,$05,$05,$05,$04,$04,$04,$04,$04,$04,$04,$04,
       $04,$04,$03,$03,$03,$03,$03,$03,$03,$03,$03,$03,$02,$02,$02,$02,
       $02,$02,$02,$02,$02,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01);

VAR SpkrTabla : ARRAY[0..255] of Byte;

{Ŀ}
{ Assembler IRQ                                                            }
{}
{$L DEVSpkr}

PROCEDURE SpkrIntHandler; FAR; EXTERNAL;

{Ŀ}
{ Name function                                                            }
{}
FUNCTION SpkrName : tDevName; FAR;
BEGIN
  SpkrName := GetString(StrDevSpkrName);
END;

{Ŀ}
{ Autodetect routine (Speaker is always present)                           }
{}
FUNCTION DevDetect : Boolean; FAR;
BEGIN
  DevDetect := True;
END;

{Ŀ}
{ Routine for adjusting the conversion table to the new output freq.       }
{}
PROCEDURE AdjustTable;
VAR I    : Word;
    TVal : Word;
BEGIN
  TVal := TimerVal SHR 1;
  For I := 0 to 255 do
    SpkrTabla[I] := (Word((SpkrBaseTabla[I]-1) * tVal) div $39) + 1;
END;

{Ŀ}
{ Device initialization routine                                            }
{}
PROCEDURE DevInit(Hz : Word); FAR;
BEGIN
  ASM
    IN      AL,61h
    OR      AL,3
    OUT     61h,AL    { Turn on speaker. }

    MOV     AL,0B6h
    OUT     43h,AL

    MOV     AL,0B0h
    OUT     43h,AL
    MOV     AL,034h
    OUT     43h,AL

    XOR     AL,AL
    OUT     42h,AL
    OUT     42h,AL
    MOV     AL,10010000b
    OUT     43h,AL    { Select Speaker timer mode. }
  END;

  CalcTimerData(Hz);    { Then do the standard polling initialization. }
  InitTimer;
  AdjustTable;          { And adjust the table for initial freq.       }
END;

{Ŀ}
{ Device deinitialization routine                                          }
{}
PROCEDURE DevEnd; FAR; ASSEMBLER;
ASM
  IN      AL,61h
  AND     AL,11111100b
  OUT     61h,AL       { Turn off speaker. }
END;

{Ŀ}
{ Accessory routines                                                       }
{}
PROCEDURE DevChgHz(Hz : Word); FAR;
BEGIN
  CalcTimerData(Hz);
  InitTimer;
  AdjustTable;
END;

PROCEDURE DevPoll; FAR;
BEGIN
END;

{Ŀ}
{ Device record                                                            }
{}
CONST SpkrData : tSoundDevice =
      (DevID : SpkrDevID;
       DMA   : False);

{Ŀ}
{ Initialization code                                                      }
{}
BEGIN
  WITH SpkrData DO
  BEGIN
    Name            := SpkrName;
    AutoDetect      := DevDetect;
    InitRut         := DevInit;
    ChgHzProc       := DevChgHz;
    GetRealFreqProc := GetRealFreq;
    TimerHandler    := SpkrIntHandler;
    PollRut         := DevPoll;
    EndRut          := DevEnd;
  END;

  InitDevice(@SpkrData);
END.
