UNIT Hardware;

INTERFACE

PROCEDURE EnableIRQ(I : Word);
PROCEDURE DisableIRQ(I : Word);
FUNCTION  SetIRQVector(I : Word; Vec : Pointer) : Pointer;

PROCEDURE DMASet(Channel, Mode : Word; Buf : Pointer; Size : Word);
PROCEDURE DMARawSet(Channel, Mode, Page : Byte; Offs, Size : Word);
PROCEDURE DMASetAsm;

IMPLEMENTATION
USES Dos;

{----------------------------------------------------------------------------}
{ PIC routines.                                                              }
{____________________________________________________________________________}

PROCEDURE EnableIRQ(I : Word);
BEGIN
  If I < 8 then Port[$21] := Port[$21] and (NOT (1 SHL i)) else
  BEGIN
    Port[$A1] := Port[$A1] AND (NOT (1 SHL (I-8)));
    Port[$21] := Port[$21] AND (NOT (1 SHL 2))
  END;
END;

PROCEDURE DisableIRQ(I : Word);
BEGIN
  If I < 8
    then Port[$21] := Port[$21] OR (1 SHL I)
    else Port[$A1] := Port[$A1] OR (1 SHL (I-8))
END;

FUNCTION SetIRQVector(I : Word; Vec : Pointer) : Pointer;
VAR P : Pointer;
    J : Word;
BEGIN
  If I < 8
    then J := I + $08
    else J := I + $68;

  GetIntVec(J, P);
  SetIntVec(J, Vec);
  SetIRQVector := P;
END;

{----------------------------------------------------------------------------}
{ DMA routines.                                                              }
{____________________________________________________________________________}

{ AH = Channel
  CH = Mode
  CL = Page
  BX = Offset
  SI = Size}

PROCEDURE DMA16Set; ASSEMBLER;
CONST PageRegTable : ARRAY[0..3] of Word = ($00, $8B, $89, $8A);
ASM
  AND     AH,3

  SHR     CL,1
  RCR     BX,1
  ADD     CL,CL

  MOV     AL,AH;
  ADD     AL,$04;
  OUT     $D4,AL     { Disable DMA channel.                         }

  XOR     AL,AL;
  OUT     $D8,AL     { Clear BYTE POINTER flip-flop to lower byte.  }
  MOV     AL,AH;
  ADD     AL,CH;
  OUT     $D6,AL     { DMA Mode register.                           }
  XOR     DH,DH
  MOV     DL,AH
  ADD     DX,DX
  ADD     DX,DX      { Calculate DMA base port.                     }
  ADD     DX,$C0

  MOV     AL,BL;
  OUT     DX,AL      { Offset of the buffer, low & high bytes.      }
  MOV     AL,BH;
  OUT     DX,AL

  PUSH    DX

  MOV     DL,AH
  ADD     DX,DX
  MOV     BX,OFFSET PageRegTable
  ADD     BX,DX      { Calculate page register port.                }
  MOV     DX,[BX]
  MOV     AL,CL;
  OUT     DX,AL      { Set DMA page.                                }

  POP     DX
  INC     DX
  INC     DX         { Calculate DMA counter port.                  }

  MOV     BX,SI
  MOV     AL,BL;
  OUT     DX,AL      { Size of the buffer minus 1, low & high byte. }
  MOV     AL,BH;
  OUT     DX,AL
  MOV     AL,AH;
  OUT     $D4,AL     { Enable DMA channel.                          }
END;

PROCEDURE DMA8Set; ASSEMBLER;
CONST PageRegTable : ARRAY[0..3] of Word = ($87, $83, $81, $82);
ASM
  MOV  AL,AH;
  ADD  AL,$04;
  OUT  $A,AL     { Disable DMA channel.                         }
  XOR  AL,AL;
  OUT  $C,AL     { Clear BYTE POINTER flip-flop to lower byte.  }
  MOV  AL,AH;
  ADD  AL,CH;
  OUT  $B,AL     { DMA Mode register.                           }
  XOR  DH,DH
  MOV  DL,AH
  ADD  DX,DX     { Calculate DMA base port.                     }

  MOV  AL,BL;
  OUT  DX,AL     { Offset of the buffer, low & high bytes.      }
  MOV  AL,BH;
  OUT  DX,AL

  PUSH DX

  MOV  BX,OFFSET PageRegTable
  ADD  BX,DX     { Calculate page register port.                }
  MOV  DX,[BX]
  MOV  AL,CL;
  OUT  DX,AL     { Set DMA page.                                }

  POP  DX
  INC  DX        { Calculate DMA counter port.                  }

  MOV  BX,SI
  MOV  AL,BL;
  OUT  DX,AL     { Size of the buffer minus 1, low & high byte. }
  MOV  AL,BH;
  OUT  DX,AL
  MOV  AL,AH;
  OUT  $A,AL     { Enable DMA channel.                          }
END;

PROCEDURE DMASetAsm; ASSEMBLER;
ASM
  CMP     AH,2
  JZ      @@Fin
  CMP     AH,4
  JZ      @@Fin
  JB      @@8bit
  CALL    DMA16Set
  JMP     @@Fin

@@8bit:
  CALL   DMA8Set

@@Fin:
END;

PROCEDURE DMARawSet(Channel, Mode, Page : Byte; Offs, Size : Word); ASSEMBLER;
ASM
  MOV     AH,[Channel]
  MOV     CH,[Mode]
  MOV     CL,[Page]
  MOV     BX,[Offs]
  MOV     SI,[Size]

  CALL    DMASetAsm
END;

PROCEDURE DMASet(Channel, Mode : Word; Buf : Pointer; Size : Word);
VAR Segm : Word;
    Offs : Word;
BEGIN
  Segm := Seg(Buf^) and $F000;
  Offs := Seg(Buf^) and $0FFF;

  If Offs>65535-OFS(Buf^) then
  BEGIN
    Inc(Segm, $1000);
    Offs := Word(Offs-65536+Ofs(Buf^));
  END else Offs := Offs + Ofs(Buf^);

  Dec(Size);

  DMARawSet(Channel, Mode, Segm SHR 12, Offs, Size);
END;

END.
