UNIT Loader669;

INTERFACE

USES Objects, SongUnit;

PROCEDURE Load669FileFormat(VAR Song : tSong; VAR St : tStream; VAR Header : tSongHeader);

IMPLEMENTATION
USES SongElements, SongUtils, Heaps, AsciiZ;

{----------------------------------------------------------------------------}
{ Internal definitions. Format of the files.                                 }
{____________________________________________________________________________}
TYPE t669FileMagic = Word;
CONST Magic669 = $6669;
TYPE tSizes = ARRAY[1..128] of Byte;

     t669Header = RECORD
       Magic        : t669FileMagic;
       Comment      : ARRAY[1..3, 1..36] of Char;
       NInstruments : Byte;
       NPatterns    : Byte;
       RepStart     : Byte;
       Sequence     : ARRAY[1..128] of Byte;
       Tempos       : ARRAY[1..128] of Byte;
       Lengths      : tSizes;
     END;

     t669Instrument = RECORD
       Name     : ARRAY[1..13] of Char;
       Size     : LongInt;
       RepStart : LongInt;
       RepLen   : LongInt;
     END;

     t669Pattern = ARRAY[1..64, 1..8] of
     RECORD
       CASE Byte OF
         0 : (W1 : Word; B : Byte);
         1 : (B1, B2, B3 : Byte);
     END;

PROCEDURE ProcessPatterns(VAR Song : tSong; VAR St : tStream; VAR Sizes: tSizes; Num : Word);
VAR Patt       : t669Pattern;
    FullTrack  : tFullTrack;
    Pattern    : pPattern;
    Track      : pTrack;
    Note       : tFullNote;
    C          : Byte;
    I, J, N, T : Word;
    Row, Size  : Word;
    NAdj       : Word;
    l          : LongInt;
BEGIN
  T := 1;
  For N := 1 to Num do
  BEGIN
    Pattern := Song.GetPattern(N);
    If Pattern = NIL then
    BEGIN
      Song.Status := msOutOfMemory;
      Exit;
    END;

    WITH Pattern^.Patt^ DO
    BEGIN
      NNotes := Sizes[N] + 1;
      NChans := Song.NumChannels;
      Tempo  := 0;
      BPM    := 0;
    END;

    St.Read(Patt, SizeOf(Patt));

    If St.Status <> stOk then
    BEGIN
      Song.Status := msFileTooShort;
      Exit;
    END;

    For J := 1 to Song.NumChannels do
    BEGIN
      FillChar(FullTrack, SizeOf(FullTrack), 0);

      For I := 1 to 64 do
        WITH FullTrack[I-1], Patt[I][J] DO
        BEGIN
          If B1 < $FE then
          BEGIN
            Period     := PeriodArray[B1 SHR 2];
            Instrument := ((Swap(W1) SHR 4) AND 63)+1;
          END;

          If B1 < $FF then Volume := ((B2 AND 15) SHL 2) + ((b2 AND 15) SHR 2)+1;

          Parameter  := B3 AND 15;
          Command    := mcNone;

          If Parameter <> 0 then
            CASE b3 SHR 4 OF
              0 : Command := mcTPortUp;
              1 : Command := mcTPortDown;
              2 : Command := mcNPortamento;
              3 : Inc(Period);
              4 : BEGIN
                Command   := mcVibrato;
                Parameter := (Parameter SHL 4) + 1
              END;
              5 : Command := mcSetTempo;
            END;
        END;

        Track := Song.GetTrack(t);
        If Track = NIL then
        BEGIN
          Song.Status := msOutOfMemory;
          Exit;
        END;

        Track^.SetFullTrack(FullTrack);
        Pattern^.Patt^.Channels[J] := T;

        Inc(T);
      END;
    END;
END;

PROCEDURE ProcessInstruments(VAR Song : tSong; VAR St : tStream; Num : Word);
VAR Instrument : tInstrumentRec;
    Instr      : pInstrument;
    Instr669   : t669Instrument;
    I, W       : Word;
    Signo      : LongInt;
    NoSigno    : LongInt;
BEGIN
  For I := 1 to Num do
    WITH Instrument DO
    BEGIN
      FillChar(Instrument, SizeOf(Instrument), 0);

      Instr := Song.GetInstrument(I);
      If Instr = NIL then
      BEGIN
        Song.Status := msOutOfMemory;
        Exit;
      END;

      St.Read(Instr669, SizeOf(Instr669));
      Instr^.SetName(StrASCIIZ(Instr669.Name, 13));
      Len  := Instr669.Size;

      If Len > 0 then
      BEGIN
        If Instr669.RepLen <= Len then
        BEGIN
          Reps := Instr669.RepStart;
          Repl := Instr669.RepLen;
        END else
        BEGIN
          Reps := 0;
          Repl := 0;
        END;

        Vol  := 64;

        If Repl        > Len then Repl := Len;
        If Reps + Repl > Len then Repl := Len - Reps;

        Instr^.Change(@Instrument);
      END else Instr^.Change(NIL);
    END;
END;

PROCEDURE ProcessSamples(VAR Song : tSong; VAR St : tStream; Num : Word);
VAR Instr : pInstrument;
    I, W  : Word;
BEGIN
  For I := 1 to Num do
  BEGIN
    Instr := Song.GetInstrument(I);

    If (Instr^.Instr <> NIL) and (Instr^.Instr^.Len > 0) then
      WITH Instr^.Instr^ DO
      BEGIN
        If Len <= MaxSample then
        BEGIN
          FullHeap.HGetMem(Pointer(Data), Len);
          If Data = NIL then
          BEGIN
            Song.Status := msOutOfMemory;
            Exit;
          END;

          St.Read(Data^, Len);

          If St.Status <> stOk then
          BEGIN
            Song.Status := msFileDamaged;
            Exit;
          END;

          For W := 0 to Len - 1 do
            Inc(Data^[W], 128);

        END else
        BEGIN
          FullHeap.HGetMem(Pointer(Data), MaxSample);
          FullHeap.HGetMem(Pointer(Xtra), Len-MaxSample);

          If (Data = NIL) or (Xtra = NIL) then
          BEGIN
            Song.Status := msOutOfMemory;
            Exit;
          END;

          St.Read(Data^, MaxSample);
          St.Read(Xtra^, Len-MaxSample);

          If St.Status <> 0 then
          BEGIN
            Song.Status := msFileDamaged;
            Exit;
          END;
        END;
      END;
  END;
END;

PROCEDURE Load669FileFormat(VAR Song : tSong; VAR St : tStream; VAR Header : tSongHeader);
VAR Hdr        : t669Header ABSOLUTE Header;
    InitialPos : LongInt;
    I          : Word;
BEGIN
  Song.FileFormat := mffComposer669;
  InitialPos := St.GetPos;

  St.Seek(InitialPos + SizeOf(t669Header));

  If Hdr.Magic <> Magic669 then
  BEGIN
    Song.Status := msNotLoaded;
    Exit;
  END;

  Song.Status := msOK;
  Song.Name := FullHeap.HNewStr(Song.FileName);
  Song.InitialTempo := 4;
  Song.InitialBPM   := 80;
  Song.Volume       := 255;
  Song.NumChannels  := 8;

  Song.SequenceLength := 0;
  For I := 1 to 128 do
    If Hdr.Sequence[I] < 128 then Song.SequenceLength := I;

  Song.SequenceRepStart := Hdr.RepStart + 1;
  Move(Hdr.Sequence, Song.PatternSequence^, Song.SequenceLength);
  Move(Hdr.Tempos,   Song.PatternTempos^,   128);

  For I := 1 to Song.SequenceLength do
    Inc(Song.PatternSequence^[I]);

  { Processing of the instruments }
  ProcessInstruments(Song, St, Hdr.NInstruments);
  If Song.Status > msOk then Exit;

  { Processing of the patterns (the partiture) }
  ProcessPatterns(Song, St, Hdr.Lengths, Hdr.NPatterns);
  If Song.Status > msOk then Exit;

  { Processing of the samples }
  ProcessSamples(Song, St, Hdr.NInstruments);
  If Song.Status > msFileTooShort then Exit;
END;

END.
