UNIT ModCommands;

INTERFACE
USES SongUnit, SongElements;

{ Configuration. }

CONST MyLoopMod          : Boolean = False;  { If True, then the MOD will loop if it was so defined.  }
      PermitFilterChange : Boolean = False;  { True if the partiture is allowed to change the filter. }
      BPMDivider         : Word    = 125;
      BPMIncrement       : Word    = 125;
      BPMCount           : Word    = 0;
      MyFirstPattern     : Word    = 0;
      MyRepStart         : Word    = 0;
      MySongLen          : Word    = 0;
      FirstPattern       : Word    = 0;
      RepStart           : Word    = 0;
      SongLen            : Word    = 0;

{ Values common to all the channels. }

CONST FilterIsOn : Boolean = False;  { Position of the filter (False = OFF).                  }
      Tempo      : Byte    = 6;      { Number of ticks in the current note.                   }

{ Values set from outside this UNIT, apart from this one. }

CONST NextNote : Word = 1; { Next note in the pattern.                              }
      NextSeq  : Word = 1; { Next pattern index (for the next note).                }
                           { They both must have been set BEFORE calling this UNIT. }

      TempoCt  : Byte = 0; { Number of the actual tick. Not changed in this UNIT.   }

{ General definition of the state of a channel. }

{ Channel state definition. }
TYPE pCanal = ^tCanal;
     tCanal = RECORD
       Note       : tFullNote;      { Note being played in the channel.   }
       Instrument : PInstrumentRec; { Pointer to the instrument data.     }
       Volume     : Byte;           { Actual volume (0 - 64).             }
       Period     : Word;           { Actual Period.                      }
       RealPeriod : Word;           { Actual adjusted Period.             }

       PeriodIncr,                  { Note portamento increment.          }
       PeriodDest : Integer;        { Note portamento destination.        }

       Arpct      : Byte;           { Arpeggio count.                     }
       Arp0,                        { Arpeggio 1st Period.                }
       Arp1,                        { Arpeggio 2nd Period.                }
       Arp2       : Word;           { Arpeggio 3rd Period.                }

       VibWave,                     { Vibrato wave form.                  }
       VibPos,                      { Vibrato position.                   }
       VibWidth,                    { Vibrato width (period).             }
       VibDepth   : Byte;           { Vibrato depth (amplitude).          }
       VibReset   : Boolean;        { Vibrato reset.                      }

       TPortaIncr : Integer;        { Tone portamento increment.          }
       VolumeIncr : ShortInt;       { Volume increment.                   }
       TPortIsFine: Boolean;
       VSldIsFine : Boolean;
  END;

PROCEDURE DoTickCommand;
PROCEDURE CommandStart(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);

IMPLEMENTATION
USES SongUtils, SoundDevices;

PROCEDURE StartArpeggio(VAR Song : tSong; VAR Can : tCanal; VAR n : tFullNote);
VAR F, G : Integer;
BEGIN
  Can.Arpct := 0;
  F := N.Period;
  If F = 0 then F := Can.Note.Period;

  Can.Arp0 := F;
  G := Word((N.Parameter SHR  4));
  Can.arp1 := PeriodArray[(NoteIdx[F] AND $FF) + G];
  G := Word((N.Parameter AND $F));
  Can.arp2 := PeriodArray[(NoteIdx[F] AND $FF) + G];
END;

PROCEDURE TickArpeggio; ASSEMBLER;
ASM
  INC     tCanal([SI]).arpct
  MOV     AL,tCanal([SI]).arpct
  DEC     AL
  JNZ     @@2
  MOV     AX,tCanal([SI]).arp0
  MOV     tCanal([SI]).Period,AX
  JMP     @@Fin

@@2:
  DEC     AL
  JNZ     @@3
  MOV     AX,tCanal([SI]).arp1
  MOV     tCanal([SI]).Period,AX
  JMP     @@Fin

@@3:
  MOV     AX,tCanal([SI]).arp2
  MOV     tCanal([SI]).Period,AX
  MOV     tCanal([SI]).arpct,0
@@Fin:
END;

PROCEDURE StartFinePortaUp(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote); FORWARD;
PROCEDURE StartFinePortaDown(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote); FORWARD;
PROCEDURE StartTPortUp(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Parameter <> 0 then
  BEGIN
    Can.TPortaIncr  := Word(N.Parameter);
    Can.TPortIsFine := False;
  END else
  If Can.tPortIsFine then
  BEGIN
    Can.Note.Command := mcFinePortaUp;
    StartFinePortaUp(Song, Can, N);
  END;
END;

PROCEDURE StartTPortDown(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Parameter <> 0 then
  BEGIN
    Can.TPortaIncr  := Word(N.Parameter);
    Can.TPortIsFine := False;
  END else
  IF Can.TPortIsFine then
  BEGIN
    Can.Note.Command := mcFinePortaDn;
    StartFinePortaDown(Song, Can, N);
  END;
END;

PROCEDURE TickTPortUp; ASSEMBLER;
ASM
  MOV     AX,tCanal([SI]).Note.Period
  SUB     AX,tCanal([SI]).TPortaIncr
  MOV     tCanal([SI]).Note.Period,AX
  MOV     tCanal([SI]).Period,AX
END;

PROCEDURE TickTPortDown; ASSEMBLER;
ASM
  MOV     AX,tCanal([SI]).Note.Period
  ADD     AX,tCanal([SI]).TPortaIncr
  MOV     tCanal([SI]).Note.Period,AX
  MOV     tCanal([SI]).Period,AX
END;

PROCEDURE StartNPortamento(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Period <> 0
    then Can.PeriodDest := N.Period
    else If Can.PeriodDest = 0 then Can.PeriodDest := Can.Period;

  If N.Parameter = 0 then
  BEGIN
    If Can.PeriodIncr > 0
      then N.Parameter := Byte( Can.PeriodIncr)
      else N.Parameter := Byte(-Can.PeriodIncr);
  END;

  If Integer(Can.PeriodDest - Can.Period) >= 0
    then Can.PeriodIncr :=  Integer(N.Parameter)
    else Can.PeriodIncr := -Integer(N.Parameter);

  Can.Note.Period := Can.Period;
END;

PROCEDURE TickNPortamento; ASSEMBLER;
ASM
  MOV     AX,tCanal([SI]).PeriodDest
  AND     AX,AX
  JZ      @@Fin
  MOV     AX,tCanal([SI]).PeriodIncr
  AND     AX,AX
  JZ      @@Fin
  MOV     BX,tCanal([SI]).Note.Period
  ADD     BX,AX
  TEST    AH,80h
  JNZ     @@neg
  CMP     BX,tCanal([SI]).PeriodDest
  JA      @@pneg
  JMP     @@cnt

@@neg:
  TEST    BH,80h
  JNZ     @@pneg
  CMP     BX,tCanal([SI]).PeriodDest
  JAE     @@cnt

@@pneg:
  MOV     BX,tCanal([SI]).PeriodDest

@@cnt:
  MOV     tCanal([SI]).Note.Period,BX
  MOV     tCanal([SI]).Period,BX

@@Fin:
END;

PROCEDURE StartVibrato(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
VAR F : Word;
BEGIN
  F := N.Parameter AND $F;
  If F <> 0 then
  CASE Can.VibWave OF
    0 : Can.VibWidth := F;
    1 : Can.VibWidth := F;
    2 : Can.VibWidth := (F*255) SHR 7;
  END;

  F := N.Parameter SHR  4;
  If F <> 0 then Can.VibDepth := F SHL 2;

  If (N.Period <> 0) and can.VibReset then Can.VibPos := 0;
END;

PROCEDURE TickVibrato; ASSEMBLER;
{ Sinus Tabla for the vibrato. }
CONST VibTabla : ARRAY[0..31] OF Byte =
      (  0, 24, 49, 74, 97,120,141,161,
       180,197,212,224,235,244,250,253,
       255,253,250,244,235,224,212,197,
       180,161,141,120, 97, 74, 49, 24);
ASM
  MOV     AH,tCanal([SI]).VibWave
  MOV     DH,tCanal([SI]).VibWidth
  MOV     DL,tCanal([SI]).VibPos
  MOV     AL,DL
  AND     AH,AH
  JNZ     @@nosn
  SHR     AL,2
  AND     AL,1Fh
  MOV     BX,OFFSET VibTabla
  XLAT
  JMP     @@set

@@nosn:
  DEC     AH
  JNZ     @@notr
  SHL     AL,1
  JNC     @@set
  NOT     AL

@@set:
  MUL     DH
  SHL     AX,1
  MOV     AL,AH
  XOR     AH,AH
  JMP     @@calc

@@notr:
  MOV     AL,DH
  XOR     AH,AH

@@calc:
  MOV     BX,tCanal([SI]).Note.Period
  TEST    DL,80h
  JZ      @@mas
  NEG     AX

@@mas:
  ADD     BX,AX
  MOV     tCanal([SI]).Period,BX
  ADD     DL,tCanal([SI]).VibDepth
  MOV     tCanal([SI]).VibPos,DL
END;

PROCEDURE TickVolSlide; FORWARD;

PROCEDURE TickT_VSlide; ASSEMBLER;
ASM
  CALL    TickNPortamento
  JMP     TickVolSlide
END;

PROCEDURE TickVib_VSlide; ASSEMBLER;
ASM
  CALL    TickVibrato
  JMP     TickVolSlide
END;

PROCEDURE TickTremolo; ASSEMBLER;
ASM
END;

PROCEDURE TickNPI1; ASSEMBLER;
ASM
END;

PROCEDURE TickSampleOffs; ASSEMBLER;
ASM
END;

PROCEDURE StartVolSlide(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Parameter <> 0 then
  BEGIN
    Can.VSldIsFine := False;
    If N.Parameter > $F
      then Can.VolumeIncr := N.Parameter SHR 4
      else Can.VolumeIncr := -ShortInt(N.Parameter AND $F);
  END else
  BEGIN
    If Can.VSldIsFine then
    BEGIN
      Can.VSldIsFine := False;
      ASM
        MOV     SI,Word PTR can
        CALL    TickVolSlide
      END;
      Can.VSldIsFine := True;
    END;
  END;
END;

PROCEDURE TickVolSlide; ASSEMBLER;
ASM
  MOV     AL,tCanal([SI]).VSldIsFine
  AND     AL,AL
  JNZ     @@Sale

  MOV     AL,tCanal([SI]).Volume
  MOV     AH,tCanal([SI]).VolumeIncr
  ADD     AL,AH
  CMP     AL,64
  JBE     @@Fin
  XOR     AL,AL
  TEST    AH,80h
  JNZ     @@Fin
  MOV     AL,64

@@Fin:
  MOV     tCanal([SI]).Volume,AL

@@Sale:
END;

PROCEDURE StartJumpPattern(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  NextSeq := N.Parameter;
  If NextSeq >= MySongLen
    then NextNote := $FFFF
    else NextNote := 1;
END;

PROCEDURE StartSetVolume(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Parameter > 64 then N.Parameter := 64;
  Can.Volume := N.Parameter;
END;

PROCEDURE StartEndPattern(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If NextNote <> 1 then Inc(NextSeq);
  If NextSeq > MySongLen
    then NextNote := $FFFF
    else NextNote := (N.Parameter AND $0F) + (N.Parameter SHR 4)*10 + 1;
  If NextNote <> $FFFF then NextNote := 1;
END;

PROCEDURE StartSetTempo(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Parameter > $30
    then BPMIncrement := N.Parameter
    else If N.Parameter <> 0  then Tempo := N.Parameter;
END;

PROCEDURE StartSetFilter(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If PermitFilterChange then FilterIsOn := N.Parameter <> 0;
END;

PROCEDURE TickSetFilter; ASSEMBLER;
ASM
END;

PROCEDURE StartFinePortaUp(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Parameter <> 0 then
  BEGIN
    Can.TPortaIncr  := Word(N.Parameter);
    Can.TPortIsFine := True;
  END else
  If NOT Can.TPortIsFine then
  BEGIN
    Can.Note.Command := mcTPortUp;
  END;

  If can.TPortIsFine then
  ASM
    MOV       SI,Word PTR [can]
    CALL      TickTPortUp
  END;
END;

PROCEDURE StartFinePortaDown(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Parameter <> 0 then
  BEGIN
    Can.TPortaIncr  := Word(N.Parameter);
    Can.TPortIsFine := True;
  END else
  If NOT Can.TPortIsFine then Can.Note.Command := mcTPortDown;

  If Can.TPortIsFine then
  ASM
    MOV       SI,Word PTR [can]
    CALL      TickTPortDown
  END;
END;

PROCEDURE TickFPortUpDown; ASSEMBLER;
ASM
END;

PROCEDURE TickGlissCtrl; ASSEMBLER;
ASM
END;

PROCEDURE TickVibCtrl; ASSEMBLER;
ASM
END;

PROCEDURE TickFineTune; ASSEMBLER;
ASM
END;

PROCEDURE TickJumpLoop; ASSEMBLER;
ASM
END;

PROCEDURE TickTremCtrl; ASSEMBLER;
ASM
END;

PROCEDURE TickNPI2; ASSEMBLER;
ASM
END;

PROCEDURE TickRetrigNote; ASSEMBLER;
ASM
END;

PROCEDURE StartVolFineUp(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Parameter <> 0 then
  BEGIN
    Can.VSldIsFine := True;
    Can.VolumeIncr := N.Parameter AND $F;
  END;

  If Can.VSldIsFine then
  BEGIN
    Can.VSldIsFine := False;
    ASM
      MOV     SI,Word PTR can
      CALL    TickVolSlide
    END;
    Can.VSldIsFine := True;
  END;
END;

PROCEDURE StartVolFineDown(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
BEGIN
  If N.Parameter <> 0 then
  BEGIN
    Can.VSldIsFine := True;
    Can.VolumeIncr := -ShortInt(N.Parameter AND $F);
  END;

  If Can.VSldIsFine then
  BEGIN
    Can.VSldIsFine := False;
    ASM
      MOV     SI,Word PTR can
      CALL    TickVolSlide
    END;
    Can.VSldIsFine := True;
  END;
END;

PROCEDURE TickVolFineUpDn; ASSEMBLER;
ASM
  JMP     TickVolSlide
END;

PROCEDURE TickNoteCut; ASSEMBLER;
ASM
END;

PROCEDURE TickNoteDelay; ASSEMBLER;
ASM
END;

PROCEDURE TickPattDelay; ASSEMBLER;
ASM
END;

PROCEDURE TickFunkIt; ASSEMBLER;
ASM
END;

PROCEDURE TickNone; ASSEMBLER;
ASM
END;

PROCEDURE StartOktArp(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
VAR F, G : Integer;
BEGIN
  Can.arpct := 0;

  F := N.Period;
  If F = 0 then F := Can.Note.Period;

  Can.arp1 := f;
  G := Word((N.Parameter SHR  4));
  Can.Arp0 := PeriodArray[(NoteIdx[F] AND $FF) - G];
  G := Word((N.Parameter AND $F));
  Can.Arp2 := PeriodArray[(NoteIdx[F] AND $FF) + G];
END;

PROCEDURE TickArpeggio2; ASSEMBLER;
ASM
  INC     tCanal([SI]).arpct
  MOV     AL,tCanal([SI]).arpct
  DEC     AL
  JNZ     @@2
  MOV     AX,tCanal([SI]).arp1
  MOV     tCanal([SI]).Period,AX
  JMP     @@Fin

@@2:
  DEC     AL
  JNZ     @@3
  MOV     AX,tCanal([SI]).arp2
  MOV     tCanal([SI]).Period,AX
  JMP     @@Fin

@@3:
  DEC     AL
  JNZ     @@4
  MOV     AX,tCanal([SI]).arp1
  MOV     tCanal([SI]).Period,AX
  JMP     @@Fin

@@4:
  MOV     AX,tCanal([SI]).arp0
  MOV     tCanal([SI]).Period,AX
  MOV     tCanal([SI]).arpct,0
@@Fin:
END;

CONST TickCommOfs : ARRAY[mcNone..mcLast] of Word =
      (OFS(TickNone),       { 0 00 }
       OFS(TickArpeggio),   { 0 xx }
       OFS(TickTPortUp),    { 1 xx }
       OFS(TickTPortDown),  { 2 xx }
       OFS(TickNPortamento),{ 3 xy }
       OFS(TickVibrato),    { 4 xy }
       OFS(TickT_VSlide),   { 5 xy }
       OFS(TickVib_VSlide), { 6 xy }
       OFS(TickTremolo),    { 7 xy }
       OFS(TickNPI1),       { 8 xx }
       OFS(TickSampleOffs), { 9 xx }
       OFS(TickVolSlide),   { A xy }
       OFS(TickNone),       { B xx }
       OFS(TickNone),       { C xx }
       OFS(TickNone),       { D xx }
       OFS(TickNone),       { E xy }
       OFS(TickNone),       { F xx }

       OFS(TickSetFilter),  { E 0x }
       OFS(TickFPortUpDown),{ E 1x }
       OFS(TickFPortUpDown),{ E 2x }
       OFS(TickGlissCtrl),  { E 3x }
       OFS(TickVibCtrl),    { E 4x }
       OFS(TickFineTune),   { E 5x }
       OFS(TickJumpLoop),   { E 6x }
       OFS(TickTremCtrl),   { E 7x }
       OFS(TickNPI2),       { E 8x }
       OFS(TickRetrigNote), { E 9x }
       OFS(TickVolFineUpDn),{ E Ax }
       OFS(TickVolFineUpDn),{ E Bx }
       OFS(TickNoteCut),    { E Cx }
       OFS(TickNoteDelay),  { E Dx }
       OFS(TickPattDelay),  { E Ex }
       OFS(TickFunkIt),     { E Fx }

       OFS(TickArpeggio),   { Okt  }
       OFS(TickArpeggio2),  { Okt  }

       OFS(TickNone));      { 0 00 }

PROCEDURE DoTickCommand; ASSEMBLER;
ASM
  CMP     BX,mcLast*2
  JNC     @@1
  ADD     BX,OFFSET TickCommOfs
  CALL    Word PTR [BX]
@@1:
END;

PROCEDURE CommandStart(VAR Song : tSong; VAR Can : tCanal; VAR N : tFullNote);
CONST F : Integer = 0;
      G : Integer = 0;
BEGIN
  If (Can.Note.Command  = mcArpeggio) or
   (((Can.Note.Command  = mcVibrato) or
     (Can.Note.Command  = mcVib_VSlide)) and
    ((       N.Command <> mcVibrato) and
     (       N.Command <> mcVib_VSlide))) then Can.Period := Can.Note.Period;

  If (N.Period <> 0) then Can.Note.Period := N.Period;

  Can.Note.Command   := N.Command;
  Can.Note.Parameter := N.Parameter;

  CASE N.Command OF
    mcArpeggio     : StartArpeggio(Song, Can, N);
    mcTPortUp      : StartTPortUp(Song, Can, N);
    mcTPortDown    : StartTPortDown(Song, Can, N);
    mcNPortamento  : StartNPortamento(Song, Can, N);
    mcVibrato      : StartVibrato(Song, Can, N);
    mcJumpPattern  : StartJumpPattern(Song, Can, N);
    mcT_VSlide,
    mcVib_VSlide,
    mcVolSlide     : StartVolSlide(Song, Can, N);
    mcSetVolume    : StartSetVolume(Song, Can, N);
    mcEndPattern   : StartEndPattern(Song, Can, N);
    mcSetTempo     : StartSetTempo(Song, Can, N);
    mcSetFilter    : StartSetFilter(Song, Can, N);
    mcVolFineUp    : StartVolFineUp(Song, Can, N);
    mcVolFineDown  : StartVolFineDown(Song, Can, N);
    mcFinePortaUp  : StartFinePortaUp(Song, Can, N);
    mcFinePortaDn  : StartFinePortaDown(Song, Can, N);
    mcOktArp       : StartOktArp(Song, Can, N);
    mcOktArp2      : StartOktArp(Song, Can, N);
  END;

  If (N.Period <> 0) AND (N.Command <> mcNPortamento) then Can.Period := N.Period;
END;

END.
