{
                     Writing (Graphic) VGA Intros/Loaders
                                     By
                                Fred Nietzche
                                  07/30/91


     I'm assuming that you don't understand the concept of video memory, so 
I'll start off from the basics then.  Video memory is defined (for our own 
purposes) as the memory that the video board scans in determining what signals
(analog) to send to the monitor.  In graphics mode, the segment of the location
of this is determined by what type of video mode you are on -
VGA/EGA modes = $A000 ($=hex), CGA modes = $B800, and Herc = $B000.  The
offsets always start off at $0.

     Now, what type of organization are the pixels (since we're in graphics 
mode) stored in?  Because of the simplicity of Mode $13 (320x200x256), most
VGA intros are written using this.  I'm not about to go ahead and explain to
you the other VGA modes dealing in the color planes, EGA/VGA registers,etc in
the higher resolution modes, there are a couple of good books out there that
can handle those of you interested in that.  Try your local bookstores and look 
for Power Graphics Programming by Michael Abrash, which contains his articles 
about higher resolution VGA programming from Programmer's Journal.  HIGHLY 
recommendable.  In any case, in this mode, each pixel on the screen 
corresponds to exactly 1 byte on the video memory.  For example, location
(0,0) = Mem[$A000:0], (1,0) = Mem[$A000:1], (2,0) = Mem[A000:2], and so on.
Fairly easy eh?  Because the memory map is linear, the next line would just be
the next byte AFTER the previous line's last pixel.  For example, location
(0,319) = Mem[$A000:319], AND THEN location (1,0) = Mem[$A000:320].  And the 
formula for determining the video memory location is

     Video Mem Offset = YPos*320 + XPos

     The actual color of the byte values stored in the video memory is
flexible and can be changed to any color of the 256,000 palette of the VGA.
This can be accomplished by altering the VGA Video DAC registers at port
addresses $3C6 through $3C9.  To read the current settings, set the Table
Read Index (port $3C7) to the color value you want, and then read the three
values from the Table Data Register (port $3C9) (one each for Red, Green, and
Blue).  Once three values have been read from that port, the current read
index is incremented by one and the next three 6 bit (range of 2^6, or 0 to 63)
values read are for the next color value.  Writing the to the Video DAC is
similar, except the Table Write Index is port $3C8.  Again, after writing
three successive 6 bit values to the Table Data Registers increments the Write
Index by one.  By the way, all the reference information about the Video DAC's
can be obtained from any EGA/VGA reference book.  I recommend getting
Advanced Programmer's Guide to the EGA/VGA by George Sutty and Steve Blair.

     Waiting for the vertical retrace start signal before updating the video
memory is the key to smooth animation (with a quick memory update too of
course).  The idea is to have the screen changes "pop up" before your eyes, and
to do this, you need to make your changes just before the the screen trace
of the video memory occurs.  For this reason, updating the video memory had
better fast enough, and generally this is the part done in assembly.  The
important port and bit locations, and a sample implementation are as follows:

       In Pascal:

         Repeat Until (Port[$3DA] And $08) = 0;
         Repeat Until (Port[$3DA] And $08) <> 0;

       And in Assembly:

                   MOV   DX,3DAh
         Wait:     IN    AL,DX
                   TEST  AL,08h
                   JZ    Wait
         Retr:     IN    AL,DX
                   TEST  AL,08h
                   JNZ   Retr


     And finally, how to get the characters on the screen.  I used the old
character set that's stored in BIOS because it saves time and space (instead
of capturing a graphical message).  The memory address for this is
$F000:$FA6E.  Because the character set is 8 bits by 8 bits, each character
takes up 8 bytes, each byte representing a horizontal slice of the character.
It's really not that confusing after fiddling around with it.

     There's also the special effect that palette cycling can do.  For
example, the color bars seen on Amiga and ST programs, although on the IBM,
it's more laborous.  What you do is simply fill each and every horizontal line
along the width of the movement of the bars with a separate color value.
Then blank out (with whatever color you desire) all of those values using the
DAC registers.  To move the bars, just change the color of the values that
need to be blanked out and the ones that need to be "filled in" with the bar
colors (again, through the DAC registers).  That's about it.  You may want to
make the movement of the bars in some pattern, like in the motion of a sine
function for effects.

     Another neat idea that palette cycling can accomplish is a shifting
checkerboard.  This one is a little bit more difficult, and I'll let you go
and figure it out on your own.

     That's all there is to it in terms of the basics and effects.  I wrote
for myself a screen capturing TSR (very crude, but what it does is dump the
video memory and palette to a file, and that's all I need.  I couldn't find
any PD programs which stored their captured screens in this format,
surprisingly!) to add painted pictures to the loaders, so that may be
something you might want to pursue for yourself.  Compress the executable
you've just created and you're done.
}

Program PaletteTricks;

  Uses Crt;


  Const CGA_CharSet_Seg = $0F000;
        CGA_CharSet_Ofs = $0FA6E;
        CharLength      = 8;
        NumChars        = 256;
        VGA_Segment     = $0A000;
        NumCycles       = 200;
        Radius          = 80;

        DispStr         : String =
                '...LOADER DESIGNED BY BRIGHAM W. THORP......EAT MY SHORTS DUDE!...';

        Colors : Array[1..15*3] Of Byte =
                 (  7,  7, 63,
                   15, 15, 63,
                   23, 23, 63,
                   31, 31, 63,
                   39, 39, 63,
                   47, 47, 63,
                   55, 55, 63,
                   63, 63, 63,
                   55, 55, 63,
                   47, 47, 63,
                   39, 39, 63,
                   31, 31, 63,
                   23, 23, 63,
                   15, 15, 63,
                    7,  7, 63  );


Picture : Array[1..13776] Of Byte = 
(* Size is 287 by 48 *)
(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,241,245,245,245,245,
245,246,246,247,247,248,248,249,249,250,250,249,249,248,248,247,247,246,246,
245,245,244,244,243,243,242,242,241,241,242,242,243,244,244,245,245,246,246,
247,247,248,248,249,249,250,250,251,251,252,252,251,250,249,248,247,246,245,
246,247,251,239,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,250,246,244,244,245,245,246,246,247,247,248,248,249,249,249,249,249,
248,248,247,247,247,246,246,246,245,245,245,245,242,0,0,0,244,247,248,
248,249,248,247,246,245,242,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,244,246,
245,245,244,244,243,243,243,242,242,241,241,242,242,243,243,243,243,244,243,
244,244,245,245,246,246,247,247,248,248,249,249,250,250,251,251,250,250,249,
249,248,248,248,249,254,239,0,0,0,0,0,0,0,249,249,244,244,246,
246,247,247,248,248,249,249,250,250,251,251,250,249,248,248,247,247,246,246,
245,245,244,244,243,243,243,243,243,243,242,242,241,241,242,242,242,243,243,
243,244,244,245,245,246,244,0,0,0,0,0,0,0,0,0,242,241,245,
245,245,245,245,246,246,247,247,248,248,249,249,250,250,249,249,248,248,247,
247,246,246,245,245,244,244,243,243,242,242,241,241,242,243,244,244,245,245,
246,246,247,248,248,249,249,250,250,251,251,252,252,251,250,249,248,247,246,
245,244,244,244,245,248,251,239,0,0,0,0,0,0,0,0,0,0,0,
0,0,250,246,243,243,243,244,244,245,245,246,246,247,247,248,248,249,249,
249,249,249,248,248,247,247,247,246,246,246,245,245,243,244,0,0,0,243,
244,247,248,249,248,246,245,243,244,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
247,244,246,245,245,244,244,243,243,243,242,242,241,241,242,242,243,243,244,
244,244,244,245,245,246,246,247,247,248,248,249,249,250,250,251,251,250,250,
249,249,248,248,245,246,247,247,248,249,251,254,239,239,249,249,245,243,243,
244,245,246,246,247,247,248,248,249,249,250,250,251,251,249,249,248,248,247,
247,246,246,245,245,244,244,244,244,243,243,242,242,241,241,242,242,243,243,
243,243,244,244,245,245,246,244,247,0,0,0,0,0,0,0,0,0,243,
242,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,248,248,248,248,247,247,246,246,246,
246,246,246,246,246,246,246,246,246,246,246,246,246,247,247,248,248,248,248,
249,249,249,249,248,247,245,246,246,249,239,0,0,0,0,0,0,0,0,
0,0,0,247,242,242,244,246,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,244,246,0,0,
0,242,243,249,249,249,249,249,249,244,246,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,246,245,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,248,248,248,247,247,246,246,246,246,246,246,
246,246,246,247,247,248,248,249,249,249,249,249,249,247,247,247,251,251,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,248,247,
247,247,247,246,246,246,246,246,246,245,246,0,0,0,0,0,0,0,0,
0,244,243,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,248,248,248,248,247,247,246,
246,246,246,246,246,246,246,246,246,246,246,246,246,246,247,247,248,248,248,
248,249,249,249,249,249,249,249,249,248,247,247,250,239,0,0,0,0,0,
0,0,0,0,245,242,243,246,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,246,246,
0,0,0,241,242,249,249,249,249,249,249,246,246,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,246,246,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,248,248,248,247,247,246,246,246,246,246,
246,246,246,246,246,246,247,247,248,248,249,249,249,249,249,249,249,247,247,
248,251,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
248,247,247,247,247,247,246,246,246,246,246,246,246,0,0,0,0,0,0,
0,0,0,245,245,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,248,248,248,248,247,
247,246,246,246,246,246,246,246,246,246,246,246,246,246,246,247,247,248,248,
248,248,249,249,249,249,249,249,249,249,249,249,249,248,248,251,239,0,0,
0,0,0,0,0,248,243,245,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
246,247,0,0,0,242,241,249,249,249,249,249,249,246,247,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,246,246,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,248,248,248,247,247,246,246,246,
246,246,246,246,246,246,246,246,247,247,248,248,249,249,249,249,249,249,249,
249,249,247,248,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,248,247,247,247,247,247,246,246,246,246,246,246,246,0,0,0,0,
0,0,0,0,0,246,247,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,248,248,248,248,
247,247,246,246,246,246,246,246,246,246,246,246,246,246,246,246,246,247,247,
248,248,248,248,249,249,249,249,249,249,249,249,249,249,249,249,249,249,254,
0,0,0,0,0,0,251,245,244,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,247,247,0,0,0,243,242,249,249,249,249,249,249,247,247,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,247,246,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,248,248,248,247,247,246,
246,246,246,246,246,246,246,246,246,246,246,247,247,248,248,249,249,249,249,
249,249,249,249,249,247,247,248,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,248,248,247,247,247,247,246,246,246,246,247,0,0,
0,0,0,0,0,0,0,247,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,248,248,
248,248,247,247,246,246,246,246,246,246,246,246,246,246,246,246,246,246,247,
247,248,248,248,248,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
250,250,0,0,0,0,0,0,247,246,248,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,247,248,0,0,0,244,243,249,249,249,249,249,249,247,248,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,247,247,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,248,248,248,247,
247,246,246,246,246,246,246,246,246,246,246,246,246,246,247,248,248,249,249,
249,249,249,249,249,249,249,249,247,247,248,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,248,248,248,247,247,247,245,246,246,247,247,
0,0,0,0,0,0,0,0,0,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
248,248,248,248,247,247,246,246,246,246,246,246,246,246,246,246,246,246,246,
246,247,247,248,248,248,248,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,250,250,239,0,0,0,0,251,247,247,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,248,248,0,0,0,245,244,249,249,249,249,249,249,
248,248,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,248,247,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,248,248,248,
247,247,246,246,246,246,246,246,246,246,246,246,246,246,246,246,247,248,248,
248,249,249,249,249,249,249,249,249,249,249,249,247,248,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,248,247,247,245,245,245,246,
247,248,0,0,0,0,0,0,0,0,0,249,238,237,237,236,236,235,235,
234,234,233,233,234,234,248,249,249,249,249,249,249,247,232,232,232,232,232,
232,232,236,236,235,235,234,234,233,233,234,234,235,235,236,236,237,238,239,
238,237,237,236,236,235,235,234,234,233,234,235,235,236,236,235,249,249,249,
249,249,249,249,249,249,251,0,0,0,0,248,248,237,237,237,237,237,237,
237,237,236,236,236,235,235,234,234,233,234,235,236,237,238,239,238,237,237,
236,236,235,235,234,234,234,233,233,249,0,0,0,245,245,249,249,249,249,
249,249,248,249,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,238,238,237,237,
236,236,235,235,234,234,233,233,234,234,234,249,249,249,249,249,249,249,238,
238,237,237,236,236,235,235,234,233,233,235,235,236,239,239,238,238,237,237,
236,236,232,235,235,249,249,249,249,249,249,249,249,249,249,247,248,247,235,
235,234,234,235,235,235,235,235,234,234,235,235,236,236,237,237,238,238,249,
249,249,249,249,249,249,234,234,234,233,233,234,234,234,235,235,236,236,237,
237,238,238,248,0,0,0,0,0,0,0,0,0,238,237,237,236,236,235,
235,234,234,233,233,234,234,248,248,249,249,249,249,249,249,247,247,232,232,
232,232,232,232,232,236,236,235,235,234,234,233,233,234,234,235,235,236,237,
238,239,238,237,236,236,235,235,234,234,233,234,235,235,236,236,235,235,232,
249,249,249,249,249,249,249,248,248,0,0,0,0,248,237,237,237,237,237,
237,237,237,237,237,236,236,236,235,235,234,234,233,234,235,236,237,238,239,
238,237,237,236,236,235,235,234,234,234,233,233,0,0,0,246,245,249,249,
249,249,249,249,249,249,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,237,237,238,
238,237,237,236,236,235,235,234,234,233,233,234,252,249,249,249,249,249,249,
249,245,238,237,236,236,235,235,234,233,233,235,235,236,239,239,238,238,237,
237,236,236,235,235,235,234,234,235,249,249,249,249,249,249,249,249,249,247,
247,249,234,235,235,235,236,235,236,236,235,233,234,235,235,236,236,237,238,
245,249,249,249,249,249,249,249,252,234,233,233,234,234,235,235,235,236,236,
237,237,238,238,237,237,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,247,248,249,249,249,249,249,249,247,247,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
232,232,232,249,249,249,249,249,249,246,246,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,246,246,
249,249,249,249,249,249,249,248,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,252,252,249,249,249,249,
249,249,249,245,245,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,245,243,250,250,250,249,249,249,233,233,233,249,249,249,249,249,249,249,
249,249,247,247,249,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,245,245,249,249,249,249,249,249,249,252,252,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,246,247,249,249,249,249,249,249,
247,248,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,232,232,249,249,249,249,249,249,232,232,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
247,246,249,249,249,249,249,249,248,248,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,252,252,249,249,
249,249,249,249,249,246,245,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,248,243,250,249,249,249,249,249,249,249,249,233,233,249,249,249,249,
249,249,249,249,249,247,247,249,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,245,246,249,249,249,249,249,249,249,252,252,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,245,246,249,249,249,249,
249,249,248,249,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,243,244,232,249,249,249,249,249,249,232,232,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,247,247,249,249,249,249,249,249,248,247,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,252,252,
249,249,249,249,249,249,249,246,246,0,0,0,0,0,0,0,0,0,0,
0,0,0,251,245,244,249,249,249,249,249,249,249,249,249,249,234,234,249,
249,249,249,249,249,249,249,249,247,247,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,246,246,249,249,249,249,249,249,249,252,252,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,245,245,249,249,
249,249,249,249,249,250,0,0,0,0,0,0,0,0,0,0,0,0,0,
241,247,247,246,246,245,245,244,244,245,246,247,248,249,250,251,252,251,250,
249,249,248,247,246,245,244,242,248,249,249,249,249,249,249,235,232,0,0,
0,0,241,242,243,243,244,244,245,245,246,246,247,247,248,248,247,247,246,
246,245,245,244,244,243,243,244,244,245,249,252,0,0,0,0,0,0,0,
0,0,0,0,248,247,249,249,249,249,249,249,247,247,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
252,252,249,249,249,249,249,249,249,247,246,0,0,0,0,0,0,0,0,
0,0,0,0,0,247,246,248,249,249,249,249,249,249,249,249,251,251,0,
234,234,249,249,249,249,249,249,249,249,249,246,247,0,0,0,0,0,0,
0,0,0,0,0,0,0,246,247,249,249,249,249,249,249,249,252,252,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,244,245,
249,249,249,249,249,249,250,250,0,0,0,0,0,0,0,0,0,0,0,
0,0,242,241,246,246,245,245,244,244,245,246,247,248,248,249,250,251,252,
251,250,250,249,249,248,247,246,245,247,249,249,249,249,249,249,249,235,237,
0,0,0,0,232,242,242,243,243,244,244,245,245,246,246,247,247,248,248,
247,246,246,245,244,244,243,243,244,244,245,245,246,246,249,254,0,0,0,
0,0,0,0,0,0,248,248,249,249,249,249,249,249,246,246,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,252,251,249,249,249,249,249,249,249,247,247,0,0,0,0,0,0,
0,0,0,0,0,0,0,247,247,249,249,249,249,249,249,249,249,251,251,
0,0,0,237,235,249,249,249,249,249,249,249,249,248,244,246,0,0,0,
0,0,0,0,0,0,0,0,0,247,247,249,249,249,249,249,249,249,251,
252,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
243,244,249,249,249,249,249,249,250,251,0,0,0,0,0,0,0,0,0,
0,0,0,0,243,242,246,246,246,246,246,246,247,247,248,248,248,248,249,
249,247,246,246,246,246,246,246,246,247,248,249,249,249,249,249,249,249,234,
235,239,0,0,0,0,238,232,232,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,248,247,247,248,251,
0,0,0,0,0,0,0,0,249,248,249,249,249,249,249,249,247,247,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,251,251,249,249,249,249,249,249,249,248,247,0,0,0,0,
0,0,0,0,0,0,0,0,0,248,249,249,249,249,249,249,249,249,251,
251,0,0,0,0,0,236,236,249,249,249,249,249,249,249,249,248,244,247,
0,0,0,0,0,0,0,0,0,0,0,247,248,249,249,249,249,249,249,
249,251,251,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,243,243,249,249,249,249,249,249,251,251,0,0,0,0,0,0,0,
0,0,0,0,0,0,244,243,246,246,246,246,246,246,247,247,248,248,248,
248,249,249,248,247,246,246,246,246,246,246,246,247,248,249,249,249,249,249,
249,234,236,0,0,0,0,0,0,232,232,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
248,248,250,0,0,0,0,0,0,0,249,249,249,249,249,249,249,249,248,
247,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,250,250,249,249,249,249,249,249,249,248,248,0,0,
0,0,0,0,0,0,0,0,0,0,0,248,249,249,249,249,249,249,249,
249,251,0,0,0,0,0,0,0,236,237,249,249,249,249,249,249,249,249,
248,245,248,0,0,0,0,0,0,0,0,0,0,248,248,249,249,249,249,
249,249,249,250,250,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,242,242,249,249,249,249,249,249,251,251,0,0,0,0,0,
0,0,0,0,0,0,0,0,245,244,246,246,246,246,246,246,247,247,248,
248,248,248,249,249,249,248,247,246,246,246,246,246,246,246,247,248,249,249,
249,249,233,233,239,0,0,0,0,0,0,239,235,235,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,248,250,251,0,0,0,0,0,0,250,250,249,249,249,249,249,
249,248,248,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,250,250,249,249,249,249,249,249,249,248,248,
0,0,0,0,0,0,0,0,0,0,0,0,0,248,249,249,249,249,249,
249,249,249,251,0,0,0,0,0,0,0,0,237,237,249,249,249,249,249,
249,249,248,245,248,0,0,0,0,0,0,0,0,0,0,248,248,249,249,
249,249,249,249,249,250,250,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,241,241,249,249,249,249,249,249,251,250,0,0,0,
0,0,0,0,0,0,0,0,0,0,246,245,246,246,246,246,246,247,247,
248,248,248,248,249,249,249,249,248,248,247,247,246,246,246,246,246,246,247,
248,248,249,232,234,236,0,0,0,0,0,0,0,0,238,235,232,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,251,251,239,0,0,0,0,0,250,250,249,249,249,
249,249,249,249,248,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,250,250,249,249,249,249,249,249,249,
249,248,0,0,0,0,0,0,0,0,0,0,0,0,0,249,249,249,249,
249,249,249,249,250,250,0,0,0,0,0,0,0,0,0,238,238,249,249,
249,249,249,249,249,247,245,251,0,0,0,0,0,0,0,0,0,248,249,
249,249,249,249,249,249,249,250,250,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,242,242,249,249,249,249,249,249,250,250,0,
0,0,0,0,0,0,0,0,0,0,0,0,247,246,246,246,246,246,246,
247,247,248,248,248,248,249,249,249,249,249,249,248,248,247,246,246,246,246,
246,246,247,232,236,235,236,239,0,0,0,0,0,0,0,0,0,238,236,
236,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,250,250,0,0,0,0,0,251,251,249,
249,249,249,249,249,249,249,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,249,249,249,249,249,249,249,
249,249,250,249,0,0,0,0,0,0,0,0,0,0,0,0,0,249,250,
249,249,249,249,249,249,250,249,0,0,0,0,0,0,0,0,0,239,238,
237,249,249,249,249,249,249,249,246,249,0,0,0,0,0,0,0,0,0,
249,250,249,249,249,249,249,249,249,249,249,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,243,243,249,249,249,249,249,249,250,
249,0,0,0,0,0,0,0,0,0,0,0,0,0,247,247,246,246,246,
246,247,247,248,248,248,248,249,249,249,249,249,249,249,249,248,248,247,246,
246,246,246,232,237,237,237,238,239,0,0,0,0,0,0,0,0,0,0,
0,238,236,237,237,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,250,250,239,0,0,0,0,251,
251,249,249,249,249,249,249,250,249,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,249,249,249,249,249,
249,249,249,249,250,250,0,0,0,0,0,0,0,0,0,0,0,0,0,
250,250,249,249,249,249,249,249,249,249,0,0,0,0,0,0,0,0,0,
0,239,237,235,249,249,249,249,249,249,248,246,253,0,0,0,0,0,0,
0,0,250,250,249,249,249,249,249,249,249,249,249,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,243,244,249,249,249,249,249,
249,249,249,0,0,0,0,0,0,0,0,0,0,0,0,0,248,248,246,
246,246,246,247,247,248,248,248,248,249,249,249,249,249,249,249,249,249,248,
248,232,236,236,236,236,237,237,239,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,239,238,237,238,238,238,239,238,238,237,237,236,236,237,237,
236,236,235,235,234,234,232,249,249,249,249,249,249,249,249,251,0,0,0,
0,250,250,249,249,249,249,249,249,250,250,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,248,249,
249,249,249,249,249,249,249,249,0,0,0,0,0,0,0,0,0,0,0,
0,0,250,251,249,249,249,249,249,249,249,247,0,0,0,0,0,0,0,
0,0,0,0,239,235,249,249,249,249,249,249,249,247,250,0,0,0,0,
0,0,0,0,249,249,249,249,249,249,249,249,249,248,248,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,244,245,249,249,249,
249,249,249,248,248,0,0,0,0,0,0,0,0,0,0,0,0,0,249,
249,246,246,246,246,247,247,251,251,235,235,249,249,249,249,249,249,249,249,
236,236,236,236,236,236,236,238,239,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,239,238,238,239,238,238,237,237,236,236,236,
236,237,237,236,236,235,235,234,234,248,249,249,249,249,249,249,249,248,0,
0,0,0,250,249,249,249,248,248,249,249,251,250,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,248,
248,249,249,249,249,249,249,249,248,248,0,0,0,0,0,0,0,0,0,
0,0,0,0,251,251,249,249,249,249,249,249,247,247,0,0,0,0,0,
0,0,0,0,0,0,0,237,249,249,249,249,249,249,249,247,247,0,0,
0,0,0,0,0,0,248,248,249,249,249,249,249,249,249,248,248,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,245,245,249,
249,249,249,249,249,247,247,0,0,0,0,0,0,0,0,0,0,0,0,
0,249,249,246,246,246,247,247,248,251,251,235,236,236,249,249,249,249,249,
249,249,249,249,249,249,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,236,233,233,248,249,249,249,249,249,248,
248,0,0,0,0,249,249,248,248,248,247,247,248,251,251,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,248,248,249,249,249,249,249,249,249,248,248,0,0,0,0,0,0,0,
0,0,0,0,0,0,251,251,249,249,249,249,249,249,247,247,0,0,0,
0,0,0,0,0,0,0,0,0,237,249,249,249,249,249,249,249,247,247,
0,0,0,0,0,0,0,0,248,248,249,249,249,249,249,249,249,248,248,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,245,
246,249,249,249,249,249,249,246,246,0,0,0,0,0,0,0,0,0,0,
0,0,0,248,248,246,246,246,247,247,248,250,250,0,236,237,237,249,249,
249,249,249,249,249,249,249,248,247,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,244,244,249,249,249,249,249,
249,248,232,0,0,0,0,249,248,246,246,246,247,247,247,252,251,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,247,248,249,249,249,249,249,249,249,247,247,0,0,0,0,0,
0,0,0,0,0,0,0,0,251,250,250,249,249,249,249,249,247,246,0,
0,0,0,0,0,0,0,0,0,0,0,254,249,249,249,249,249,249,249,
248,247,254,0,0,0,0,0,0,0,247,247,249,249,249,249,249,249,249,
248,247,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,246,246,249,249,249,249,249,249,245,246,0,0,0,0,0,0,0,0,
0,0,0,0,0,247,247,246,246,246,247,247,248,250,250,0,0,237,238,
238,249,249,249,249,249,249,249,249,247,246,246,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,249,245,245,247,248,249,
249,249,249,232,232,0,0,0,0,248,248,246,246,246,246,246,246,252,252,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,247,247,249,249,249,249,249,249,249,246,246,0,0,0,
0,0,0,0,0,0,0,0,0,0,250,250,249,249,249,248,249,249,246,
246,0,0,0,0,0,0,0,0,0,0,0,0,0,248,248,249,249,249,
249,249,249,248,250,0,0,0,0,0,0,0,246,246,249,249,249,249,249,
249,249,247,247,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,246,247,249,249,249,249,249,249,245,245,0,0,0,0,0,0,
0,0,0,0,0,0,0,247,246,246,246,247,247,248,248,249,249,0,0,
0,238,237,237,249,249,249,249,249,249,249,249,246,245,245,0,0,0,0,
0,0,0,0,0,0,0,0,246,247,247,246,246,245,245,244,244,243,243,
242,241,241,242,242,243,243,244,244,245,245,246,246,247,247,247,246,247,247,
248,249,249,249,249,232,232,0,0,0,0,248,247,246,246,246,246,246,246,
252,249,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,246,247,249,249,249,249,249,249,249,245,245,0,
0,0,0,0,0,0,0,0,0,0,0,0,250,249,249,248,248,248,247,
248,246,245,0,0,0,0,0,0,0,0,0,0,0,0,0,249,247,249,
249,249,249,249,249,248,250,0,0,0,0,0,0,0,245,245,249,249,249,
249,249,249,249,247,246,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,247,247,249,249,249,249,249,249,246,245,0,0,0,0,
0,0,0,0,0,0,0,0,0,246,245,246,246,247,247,248,248,248,248,
0,0,0,0,237,236,236,249,249,249,249,249,249,249,249,245,246,246,0,
0,0,0,0,0,0,0,0,0,0,247,246,246,246,245,245,244,244,243,
243,242,242,241,241,241,242,242,243,243,244,244,245,245,246,246,247,247,246,
246,247,247,248,248,249,249,236,238,0,0,0,0,247,247,246,246,246,246,
246,246,249,249,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,246,246,249,249,249,249,249,249,249,244,
245,0,0,0,0,0,0,0,0,0,0,0,0,0,249,249,248,246,246,
246,247,247,245,245,0,0,0,0,0,0,0,0,0,0,0,0,0,246,
246,249,249,249,249,249,249,248,250,0,0,0,0,0,0,0,245,244,249,
249,249,249,249,249,248,246,246,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,247,248,249,249,249,249,249,249,246,246,0,0,
0,0,0,0,0,0,0,0,0,0,0,245,244,246,246,247,246,247,247,
247,248,0,0,0,0,0,236,235,235,249,249,249,249,249,249,249,249,246,
247,247,0,0,0,0,0,0,0,0,0,0,246,246,248,248,249,249,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,247,
247,246,247,247,247,247,247,248,237,236,239,0,0,0,0,247,246,246,246,
246,246,246,246,249,247,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,246,246,249,249,248,248,249,248,
249,244,244,0,0,0,0,0,0,0,0,0,0,0,0,0,249,247,247,
246,246,246,246,246,245,244,0,0,0,0,0,0,0,0,0,0,0,0,
0,246,247,249,249,249,249,249,249,248,250,0,0,0,0,0,0,0,244,
244,249,248,248,249,249,248,248,246,246,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,248,248,249,249,249,249,249,249,247,246,
0,0,0,0,0,0,0,0,0,0,0,0,0,244,245,246,246,247,245,
246,246,246,247,0,0,0,0,0,0,235,235,232,249,249,249,249,249,249,
249,249,247,247,247,0,0,0,0,0,0,0,0,0,246,245,248,248,249,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,247,246,246,247,247,247,247,248,237,238,0,0,0,0,0,246,246,
246,246,246,246,246,246,247,247,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,245,246,248,248,248,247,
248,248,248,243,244,0,0,0,0,0,0,0,0,0,0,0,0,0,247,
247,247,246,246,246,246,246,244,247,247,246,246,246,245,246,246,247,247,248,
248,249,248,247,248,249,249,249,249,249,249,249,254,0,0,0,0,0,0,
0,244,243,248,248,247,248,248,248,248,246,245,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,248,249,249,249,249,249,249,249,
247,247,0,0,0,0,0,0,0,0,0,0,0,0,0,245,246,246,247,
246,246,245,245,246,246,0,0,0,0,0,0,0,232,232,232,248,249,249,
249,249,249,249,249,247,248,248,0,0,0,0,0,0,0,0,245,245,248,
248,248,248,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,248,246,246,246,246,247,247,236,236,239,0,0,0,0,0,
246,245,246,246,246,246,246,246,247,246,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,245,246,248,248,
248,247,248,248,248,243,244,0,0,0,0,0,0,0,0,0,0,0,0,
0,247,247,247,246,246,246,246,246,244,247,247,246,246,246,245,246,246,247,
247,248,248,249,248,247,248,249,249,249,249,249,249,249,254,0,0,0,0,
0,0,0,244,243,248,248,247,248,248,248,248,246,245,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,249,249,249,249,249,249,
249,249,248,247,0,0,0,0,0,0,0,0,0,0,0,0,0,246,247,
246,247,247,247,245,246,245,246,0,0,0,0,0,0,0,0,232,232,235,
248,248,249,249,249,249,248,248,248,249,249,0,0,0,0,0,0,0,245,
244,247,248,248,248,249,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,247,246,246,246,247,232,235,238,0,0,0,0,
0,0,245,245,246,246,246,246,246,246,246,246,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,245,245,
248,248,247,247,247,247,248,242,243,0,0,0,0,0,0,0,0,0,0,
0,0,0,247,246,246,246,246,246,246,246,246,246,246,247,247,247,248,248,
249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,249,0,0,0,
0,0,0,0,0,243,242,248,247,247,247,247,248,248,245,245,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,249,249,249,249,
249,249,249,249,248,248,0,0,0,0,0,0,0,0,0,0,0,0,0,
247,248,246,247,246,247,246,246,245,245,0,0,0,0,0,0,0,0,0,
232,232,235,248,248,248,248,248,248,248,248,249,250,250,0,0,0,0,0,
0,244,244,247,248,248,249,249,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,248,249,249,249,249,248,247,246,246,234,235,237,0,0,0,
0,0,0,0,245,246,246,246,246,246,246,246,246,247,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
245,245,247,247,247,246,247,247,247,241,242,0,0,0,0,0,0,0,0,
0,0,0,0,0,246,246,246,246,246,246,246,246,246,246,246,247,247,247,
248,248,249,249,249,249,249,249,249,249,249,249,249,249,249,249,236,237,0,
0,0,0,0,0,0,0,242,241,247,247,246,247,247,247,247,245,245,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,249,249,
249,249,249,249,249,249,249,248,0,0,0,0,0,0,0,0,0,0,0,
0,0,248,249,246,247,247,248,247,247,246,245,0,0,0,0,0,0,0,
0,0,0,232,232,235,248,248,248,248,248,249,248,248,250,250,249,0,0,
0,0,0,244,243,248,248,248,249,249,249,249,249,249,249,249,249,249,249,
249,249,249,249,249,249,249,248,248,248,249,248,248,233,234,234,237,0,0,
0,0,0,0,0,0,246,247,246,246,246,246,246,247,247,248,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,244,245,246,246,246,246,246,246,246,242,241,0,0,0,0,0,0,
0,0,0,0,0,0,0,246,245,245,246,246,246,246,246,246,246,246,246,
247,247,248,248,248,248,249,249,249,249,249,249,249,249,249,249,249,235,236,
238,0,0,0,0,0,0,0,0,241,242,246,246,246,246,246,246,246,245,
244,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
249,234,234,235,236,237,238,238,237,249,0,0,0,0,0,0,0,0,0,
0,0,0,0,249,235,234,233,235,236,237,237,246,246,0,0,0,0,0,
0,0,0,0,0,0,232,232,235,248,248,249,249,249,249,249,248,249,249,
249,0,0,0,0,243,243,238,237,237,236,236,235,235,234,234,233,233,234,
234,235,235,236,236,237,237,236,236,235,235,234,234,233,233,233,236,239,0,
0,0,0,0,0,0,0,0,247,233,233,234,234,235,235,236,236,248,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,244,244,246,246,246,246,246,246,246,243,242,0,0,0,0,
0,0,0,0,0,0,0,0,0,245,245,246,246,246,246,246,246,246,247,
247,247,247,247,247,248,248,249,249,249,249,249,249,249,249,249,249,249,234,
233,237,0,0,0,0,0,0,0,0,0,242,243,246,246,246,246,246,246,
246,244,244,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,234,234,235,235,236,237,238,239,238,237,0,0,0,0,0,0,0,
0,0,0,0,0,0,235,234,233,234,235,236,236,237,237,246,0,0,0,
0,0,0,0,0,0,0,0,0,232,235,234,248,249,249,249,248,249,249,
247,249,249,248,0,0,0,243,239,237,237,236,236,235,235,234,234,234,233,
233,234,234,234,235,235,236,236,237,237,236,236,235,235,234,234,236,239,0,
0,0,0,0,0,0,0,0,0,0,233,233,234,234,235,235,236,236,237,
237,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,244,244,246,246,246,246,246,246,246,244,243,0,0,
0,0,0,0,0,0,0,0,0,0,0,245,245,247,246,246,246,246,247,
247,248,248,248,248,248,248,248,248,249,249,249,249,249,249,249,249,249,249,
236,235,237,0,0,0,0,0,0,0,0,0,0,243,244,246,246,246,246,
246,246,246,244,244,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,232,235,234,248,249,248,249,
248,248,248,247,248,248,248,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,244,244,246,246,246,246,246,246,246,244,243,
0,0,0,0,0,0,0,0,0,0,0,0,0,245,245,247,246,246,246,
246,247,247,248,248,248,248,248,248,248,248,249,249,249,249,249,249,249,249,
237,237,236,235,237,0,0,0,0,0,0,0,0,0,0,243,244,246,246,
246,246,246,246,246,244,244,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,235,235,234,248,
248,248,248,248,248,248,247,248,248,247,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,244,233,234,235,236,237,237,238,238,
239,244,0,0,0,0,0,0,0,0,0,0,0,0,0,245,234,234,235,
235,236,236,237,237,238,238,239,238,238,237,237,236,236,237,238,238,239,239,
238,238,238,237,238,239,0,0,0,0,0,0,0,0,0,0,0,244,244,
239,238,237,237,236,235,234,233,244,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,235,
234,233,248,248,248,248,247,247,247,247,247,247,247,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,233,234,234,235,236,236,237,
237,238,238,239,0,0,0,0,0,0,0,0,0,0,0,0,0,234,234,
235,235,236,236,237,237,238,238,239,238,238,238,237,237,236,236,237,237,238,
238,239,239,238,239,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
244,239,238,237,237,236,236,235,234,234,233,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,235,234,233,247,247,247,247,247,247,247,247,246,246,246,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,234,234,233,246,246,246,246,247,247,247,247,245,245,245,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,233,233,238,239,238,237,236,235,234,233,234,
235,245,245,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,238,239,238,237,236,236,235,234,
233,234,234,235,235,245,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);


PicPalette : Array[1..22*4] Of Byte = 
(232,31,25,23,
233,28,22,19,
234,25,19,16,
235,22,16,12,
236,19,14,9,
237,16,12,7,
238,13,10,5,
239,10,8,3,
241,46,47,63,
242,40,42,60,
243,35,38,56,
244,30,34,53,
245,25,30,50,
246,21,28,47,
247,17,25,44,
248,14,22,41,
249,11,20,38,
250,8,19,34,
251,5,16,31,
252,3,15,28,
253,1,13,25,
254,0,12,22);


Type  OneChar =Array[1..CharLength] Of Byte;


Var   CharSet : Array[1..NumChars] Of OneChar;
      Locs    : Array[1..NumCycles] Of Integer;
      BarLocs : Array[1..4] Of Integer;


  Procedure GetChars;

    Var NumCounter,
        ByteCounter,
        MemCounter   :Integer;
  
    Begin { GetChars }
      MemCounter:=0;
      For NumCounter:=1 To NumChars Do
        For ByteCounter:=1 To CharLength Do
          Begin
            CharSet[NumCounter][ByteCounter]:=Mem[CGA_CharSet_Seg:CGA_CharSet_Ofs+MemCounter];
            Inc(MemCounter);
          End;
    End;  { GetChars }


  Procedure VideoMode ( Mode : Byte );

    Begin { VideoMode }
      Asm
        Mov  AH,00
        Mov  AL,Mode
        Int  10h
      End;
    End;  { VideoMode }


  Procedure SetColor ( Color, Red, Green, Blue : Byte );

    Begin { SetColor }
      Port[$3C8] := Color;
      Port[$3C9] := Red;
      Port[$3C9] := Green;
      Port[$3C9] := Blue;
    End;  { SetColor }


  Procedure DispVert ( Var CurrLine : Integer );

    Var Letter : OneChar;
        VertLine,
        Count  : Integer;

    Begin { DispVert }
      Letter := CharSet[Ord(DispStr[(CurrLine Div 8)+1])+1];
      VertLine := (CurrLine-1) Mod 8;
      For Count := 1 To 8 Do
        If Letter[Count] And ($80 Shr VertLine) = 0
          Then Mem[VGA_Segment:185*320+(Count-1)*320+319] := 0
          Else Mem[VGa_Segment:185*320+(Count-1)*320+319] := 181;
    End;  { DispVert }


  Procedure CalcLocs;

    Var Count : Integer;

    Begin { CalcLocs }
      For Count := 1 To NumCycles Do
        Locs[Count] := Round(Radius*Sin((2*Pi/NumCycles)*Count))+Radius+1;
    End;  { CalcLocs }


  Procedure DrawBar ( Var Bar : Integer );

    Var Count,
        PrevBar : Integer;

    Begin { DrawBar }
      If BarLocs[Bar] = 1
        Then PrevBar := Locs[NumCycles]
        Else PrevBar := Locs[BarLocs[Bar]-1];
      Port[$3C8] := PrevBar;
      For Count := 1 To 15*3 Do
        Port[$3C9] := 0;
      Port[$3C8] := Locs[BarLocs[Bar]];
      For Count := 1 To 15*3 Do
        Port[$3C9] := Colors[Count];
      Inc(BarLocs[Bar]);
    End;  { DrawBar }


  Var CurrVert,
      XCount,
      Count     : Integer;
      Key       : Char;
      MemPos    : Word;

  Begin { PaletteTricks }

    VideoMode($13);
    Port[$3C8] := 1;
    For Count := 1 To 180 Do
      SetColor(Count,0,0,0);
    For Count := 1 To 23 Do
      Begin
        Port[$3C8] := PicPalette[(Count-1)*4+1];
        Port[$3C9] := PicPalette[(Count-1)*4+2];
        Port[$3C9] := PicPalette[(Count-1)*4+3];
        Port[$3C9] := PicPalette[(Count-1)*4+4];
      End;
    MemPos := 0;
    For Count := 1 To 180 Do
      Begin
        FillChar(Mem[VGA_Segment:MemPos],320,Chr(Count));
        MemPos := MemPos + 320;
      End;

    For Count := 1 To 48 Do
      For XCount := 1 To 287 Do
        If Picture[(Count-1)*287+XCount] <> 0
          Then Mem[VGA_Segment:50*320+(Count-1)*320+20+XCount] := Picture[(Count-1)*287+XCount];

    SetColor(181,63,63,0);
    CalcLocs;
    For Count := 1 To 4 Do
      BarLocs[Count] := Count*10;

    GetChars;
    CurrVert := 1;
    Repeat
      Repeat Until (Port[$3DA] And $08) = 0;
      Repeat Until (Port[$3DA] And $08) <> 0;
      For Count := 1 To 4 Do
        Begin
          DrawBar(Count);
          If BarLocs[Count] > NumCycles Then BarLocs[Count] := 1;
        End;

      For Count := 1 To 8 Do
        Move(Mem[VGA_Segment:185*320+(Count-1)*320+1],
             Mem[VGA_Segment:185*320+(Count-1)*320],319);
      DispVert(CurrVert);
      Inc(CurrVert);
      If CurrVert > Length(DispStr) * 8
        Then CurrVert := 1;

    Until Keypressed;
    Key := ReadKey;
    VideoMode(3);
  End.  { PaletteTricks }
