UNIT SongUtils;

INTERFACE
USES SongUnit, SongElements;

{----------------------------------------------------------------------------}
{ Definitions for accelerating the use of note periods.                      }
{____________________________________________________________________________}

CONST NumberOctaves = 7;
      NumberNotes   = 12;
      NumberPeriods = NumberOctaves * NumberNotes;

TYPE tPeriodSet = ARRAY[0..NumberOctaves-1] of         { Octave }
                  ARRAY[0..NumberNotes  -1] of Word;   { Note   }

     tPeriodArray = ARRAY[0..NumberPeriods - 1] of Word;

      { The different note values. }
CONST PeriodSet : tPeriodSet =
       {  C     C#    D     D#    E     F     F#    G     G#    A     A#    B  }
      (($06B0,$0650,$05F5,$05A0,$054F,$0503,$04BB,$0477,$0436,$03FA,$03C1,$038B),
       ($0358,$0328,$02FB,$02D0,$02A7,$0281,$025D,$023B,$021B,$01FD,$01E0,$01C5),
       ($01AC,$0194,$017D,$0168,$0154,$0141,$012F,$011E,$010E,$00FE,$00F0,$00E3),
       ($00D6,$00CA,$00BF,$00B4,$00AA,$00A0,$0097,$008F,$0087,$007F,$0078,$0071),
       ($006B,$0065,$005F,$005A,$0055,$0050,$004C,$0047,$0043,$0040,$003C,$0039),
       ($0035,$0032,$0030,$002D,$002A,$0028,$0026,$0024,$0022,$0020,$001E,$001C),
       ($001B,$0019,$0018,$0016,$0015,$0014,$0013,$0012,$0011,$0010,$000F,$000E));

      { The different inter-note values. }
      PeriodDiff : tPeriodSet =
      (($0680,$0622,$05CA,$0577,$0529,$04DF,$0499,$0456,$0418,$03DD,$03A6,$0371),
       ($0340,$0311,$02E5,$02BB,$0294,$026F,$024C,$022B,$020C,$01EE,$01D2,$01B8),
       ($01A0,$0188,$0172,$015E,$014A,$0138,$0126,$0116,$0106,$00F7,$00E9,$00DC),
       ($00D0,$00C4,$00B9,$00AF,$00A5,$009B,$0093,$008B,$0083,$007B,$0074,$006E),
       ($0068,$0062,$005C,$0057,$0052,$004E,$0049,$0045,$0041,$003E,$003A,$0037),
       ($0033,$0031,$002E,$002B,$0029,$0027,$0025,$0023,$0021,$001F,$001D,$001B),
       ($001A,$0018,$0017,$0015,$0014,$0013,$0012,$0011,$0010,$000F,$000E,$000E));

VAR PeriodArray : tPeriodArray ABSOLUTE PeriodSet;

TYPE tNoteString = String[3];

     tNoteSet       = ARRAY[0..2047] of Word;
     tNoteStringSet = ARRAY[0..NumberPeriods] of TNoteString;

VAR NoteIdx : tNoteSet;
    { For each period, specifies its closest note, in two ways:      }
    {   Hi byte: octave in the hi nibble and note in the low nibble. }
    {  Low byte: sequential note for indexing.                       }

    NoteStr : tNoteStringSet; { The strings for each note (e.g. 'A#2'). }

FUNCTION  SwapLong(L : LongInt) : LongInt;
PROCEDURE NoteFreq(F : Word; VAR S : tNoteString);

PROCEDURE InitModVideoTables;
PROCEDURE InitModUnit;

FUNCTION  FullNotesEqual(VAR FN1, FN2 : tFullNote) : Boolean;

IMPLEMENTATION

FUNCTION SwapLong(L : LongInt) : LongInt;
VAR W : ARRAY[0..1] of Word ABSOLUTE l;
    R : Word;
BEGIN
  R    := Swap(W[0]);
  W[0] := Swap(W[1]);
  W[1] := R;
  SwapLong := L;
END;

PROCEDURE NoteFreq(F : Word; VAR S : tNoteString);
BEGIN
  If f > 2047 THEN F := 2047;

  S := NoteStr[NoteIdx[F] AND $FF];
END;

{----------------------------------------------------------------------------}
{ Initialization routines.                                                   }
{____________________________________________________________________________}

PROCEDURE InitModUnit;
VAR L       : LongInt;
    F, O, I : Word;
LABEL Octava, NextFreq;
BEGIN
  For F := 0 to 2047 do
  BEGIN
    For O := 0 to 6 do
      If F > PeriodDiff[O][11] then Goto Octava;
    I := 0;
    O := 0;
    Goto NextFreq;

Octava:
    For I := 0 to 11 do
      If F > PeriodDiff[O][I] then Goto NextFreq;
    I := 0; O := 0;

NextFreq:
    NoteIdx[F] := (O*16+I)*256 + (O*12+I)
  END;
END;

PROCEDURE InitModVideoTables;
CONST NoteLet : String[12] = 'CCDDEFFGGAAB';
      NoteSus : String[12] = ' # #  # # # ';
VAR O, I : Word;
    S    : String[3];
BEGIN
  For I := 0 to 12*7-1 do
  BEGIN
    S[0] := Chr(3);
    O    := I DIV 12;
    S[3] := Chr(O + Ord('0'));
    O    := I MOD 12 + 1;
    S[1] := NoteLet[O];
    S[2] := NoteSus[O];

    NoteStr[I] := S;
  END;

  NoteStr[12*7] := '---';
END;

FUNCTION FullNotesEqual(VAR FN1, FN2 : tFullNote) : Boolean;
TYPE TFNArray = ARRAY[1..SizeOf(tFullNote)] of Byte;
VAR FNA1 : tFNArray ABSOLUTE FN1;
    FNA2 : tFNArray ABSOLUTE FN2;
    I    : Word;
BEGIN
  FullNotesEqual := False;
  For I := 1 to SizeOf(tFullNote) do
    If FNA1[I] <> FNA2[I] then Exit;
  FullNotesEqual := True;
END;

END.