UNIT SoundBlaster;
{$R-}

INTERFACE
{ I/O Port offsets. }

CONST CMS1DataPortOffset = $00;  { CM/S 1-6  Data port.             Write Only. }
      CMS1AddrPortOffset = $01;  { CM/S 1-6  Address port.          Write Only. }
      CMS2DataPortOffset = $02;  { CM/S 7-12 Data port.             Write Only. }
      CMS2AddrPortOffset = $03;  { CM/S 7-12 Address port.          Write Only. }

      MixAddrPortOffset  = $04;  { Mixer register port.             Write Only. }
      MixDataPortOffset  = $05;  { Mixer data port.                 Read/Write. }

      FMStatPortOffset   = $08;  { Mono FM Status port.             Read  Only. }
      FMAddrPortOffset   = $08;  { Mono FM Address port.            Write Only. }
      FMDataPortOffset   = $09;  { Mono FM Data port.               Write Only. }

      LFMStatPortOffset  = $00;  { Left FM Status port.             Read  Only. }
      LFMAddrPortOffset  = $00;  { Left FM Address port.            Write Only. }
      LFMDataPortOffset  = $01;  { Left FM Data port.               Write Only. }

      RFMStatPortOffset  = $02;  { Right FM Status port.            Read  Only. }
      RFMAddrPortOffset  = $02;  { Right FM Address port.           Write Only. }
      RFMDataPortOffset  = $03;  { Right FM Data port.              Write Only. }

      DSPResetPortOffset = $06;  { DSP Reset port.                  Write Only. }
      DSPReadPortOffset  = $0A;  { DSP Read data port.              Read  Only. }
      DSPLifePortOffset  = $0A;  { DSP Read data port.              Read  Only. }
      DSPWStatPortOffset = $0C;  { DSP Write buffer status port.    Write Only. }
      DSPWritePortOffset = $0C;  { DSP Write data port.             Write Only. }
      DSPRStatPortOffset = $0E;  { DSP Read buffer status port.     Read  Only. }
      DSP8AckPortOffset  = $0E;  {  8 bit DMA IRQ Acknowledge port. Write Only. }
      DSP16AckPortOffset = $0F;  { 16 bit DMA IRQ Acknowledge port. Write Only. }

      CDDataPortOffset   = $10;  { CD-ROM Data port.                Read  Only. }
      CDCmdPortOffset    = $10;  { CD-ROM Command port.             Write Only. }
      CDStatPortOffset   = $11;  { CD-ROM Status port.              Read  Only. }
      CDResetPortOffset  = $12;  { CD-ROM Reset port.               Write Only. }
      CDEnablePortOffset = $13;  { CD-ROM Enable port.              Write Only. }

{ I/O Ports. Same as above. }
      CMS1DataPort : Word = $220 + CMS1DataPortOffset;
      CMS1AddrPort : Word = $220 + CMS1AddrPortOffset;
      CMS2DataPort : Word = $220 + CMS2DataPortOffset;
      CMS2AddrPort : Word = $220 + CMS2AddrPortOffset;

      MixAddrPort  : Word = $220 + MixAddrPortOffset;
      MixDataPort  : Word = $220 + MixDataPortOffset;

      FMStatPort   : Word = $220 + FMStatPortOffset;
      FMAddrPort   : Word = $220 + FMAddrPortOffset;
      FMDataPort   : Word = $220 + FMDataPortOffset;

      LFMStatPort  : Word = $220 + LFMStatPortOffset;
      LFMAddrPort  : Word = $220 + LFMAddrPortOffset;
      LFMDataPort  : Word = $220 + LFMDataPortOffset;

      RFMStatPort  : Word = $220 + RFMStatPortOffset;
      RFMAddrPort  : Word = $220 + RFMAddrPortOffset;
      RFMDataPort  : Word = $220 + RFMDataPortOffset;

      DSPResetPort : Word = $220 + DSPResetPortOffset;
      DSPReadPort  : Word = $220 + DSPReadPortOffset;
      DSPLifePort  : Word = $220 + DSPLifePortOffset;
      DSPWStatPort : Word = $220 + DSPWStatPortOffset;
      DSPWritePort : Word = $220 + DSPWritePortOffset;
      DSPRStatPort : Word = $220 + DSPRStatPortOffset;
      DSP8AckPort  : Word = $220 + DSP8AckPortOffset;
      DSP16AckPort : Word = $220 + DSP16AckPortOffset;

      CDDataPort   : Word = $220 + CDDataPortOffset;
      CDCmdPort    : Word = $220 + CDCmdPortOffset;
      CDStatPort   : Word = $220 + CDStatPortOffset;
      CDResetPort  : Word = $220 + CDResetPortOffset;
      CDEnablePort : Word = $220 + CDEnablePortOffset;

{ Configuration. }
      SBPort       : Word    = $FFFF; { Base port. $FFFF Means Autodetect.      }
      SBIrq        : Word    = 7;     { DMA IRQ level.                          }
      SBDMAChan    : Word    = 1;     { DMA channel.                            }
      SBDefTimeout : Word    = 5000;  { Default DSP timeout.                    }
      SBHiSpeed    : Boolean = True;  { User Desires HS DMA mode if True.       }
      SBForce      : Boolean = False; { Force True the detection of the SB.     }
      MixerForce   : Boolean = False; { Force True the detection of the Mixer.  }
      SBProForce   : Boolean = False; { Force True the detection of the SB Pro. }
      SB16Force    : Boolean = False; { Force True the detection of the SB 16.  }

{ Card information. }

      SBVersionMin : Byte       = 0;
      SBVersionMaj : Byte       = 0;
      SBVersionStr : String[ 5] = '';
      SBCopyright  : String[80] = '';
      SBResponse1  : Byte       = 0;
      SBResponse2  : Byte       = 0;

VAR SBVersion : Word ABSOLUTE SBVersionMin;

{ Run-time information. }
CONST SBRegDetected : Boolean = False;
      SBRegInited   : Boolean = False;
      SBProDetected : Boolean = False;
      SBProInited   : Boolean = False;
      SB16Detected  : Boolean = False;
      SB16Inited    : Boolean = False;
      MixerDetected : Boolean = False;

      SBWorksOk         : Boolean = True;  { Set to False if DSP timeouts.         }
      HSBlockSpecified  : Word    = 0;     { Set to the last hi-speed block size.  }
      SB16BlockSpecified: Word    = 0;     { Set to the last SB 16 block size.     }
      SBStereo          : Boolean = False; { Stereo DMA mode if True.              }
      SBFilter          : Boolean = False; { SB Pro output filter ON if True.      }

      DoHiSpeed : Boolean = False; { Hi speed DMA mode if True.            }
      SB16Bit   : Boolean = False; { 16 bit output if True.                }

      TimeConst : Byte    = 0;

      DMAStart   : Boolean = False;
      DMAStop    : Boolean = False;
      DMAStopped : Boolean = False;

      DMAIrqWatch : Byte    = 0;

{ DSP Commands. }
      sdcSendOneSample  = $10;  { Send a sample to the DAC directly (mono mode only). }
      sdcStartLSpeedDMA = $14;  { Start a low-speed DMA transfer.                     }
      sdcSetTimeConst   = $40;  { Set the time constant.                              }
      sdcSetHSpeedSize  = $48;  { Set hi-speed DMA transfer length.                   }
      sdcStartHSpeedDMA = $91;  { Start a hi-speed DMA transfer.                      }
      sdcTurnOnSpeaker  = $D1;  { Turn on the SB speaker.                             }
      sdcTurnOffSpeaker = $D3;  { Turn off the SB speaker.                            }
      sdcGetDSPVersion  = $E1;  { Get the DSP version number.                         }
      sdcGetCopyright   = $E3;  { Get the card copyright string.                      }

{ Mixer registers. }
      mxrDataReset    = $00;
      mxrDACVolume    = $04;
      mxrMicMixing    = $0A;
      mxrInSetting    = $0C;
      mxrOutSetting   = $0E;
      mxrMasterVolume = $22;
      mxrFMVolume     = $26;
      mxrCDVolume     = $28;
      mxrLineVolume   = $2E;

{ Bit masks for the mixer registers. }
      mxiFilterVal = $38;
      mxiADCVal    = $06;
      mxoFilterNeg = $20;
      mxoStereoOn  = $02;

TYPE tMixerVolume =
     (mvMaster, mvVoice, mvFM, mvLine, mvMicrophone, mvSpeaker, mvCD);

CONST SBProRegs : ARRAY[mvMaster..mvCD] of Byte = ($22, $04, $26, $2E, $0A, $00, $28);
      SB16Regs  : ARRAY[mvMaster..mvCD] of Byte = ($30, $32, $34, $38, $3A, $3B, $34);

{ SB basic }
FUNCTION  SBReset : Boolean;

PROCEDURE SBWriteLoop(T : Word);
PROCEDURE SBWriteByte(T : Word; B : Byte);
PROCEDURE SBReadLoop(T : Word);
FUNCTION  SBReadByte(T : Word) : Byte;

{ Mixer basic }
PROCEDURE SBWriteMixerReg(Reg, Val : Byte);
FUNCTION  SBReadMixerReg(Reg : Byte) : Byte;

{ SB Reg }
FUNCTION  SBRegDetect : Boolean;
PROCEDURE SBRegInit;
PROCEDURE SBRegDone;

PROCEDURE SBGetDSPVersion;
PROCEDURE SBGetCopyrightString;
PROCEDURE SBSetTimeConst(TC : Byte);
PROCEDURE SBUpdateTimeConst;
PROCEDURE SBStartSampleLS(Len : Word; Cont : Boolean);
PROCEDURE SBStartSampleHS(Len : Word; Cont : Boolean);
PROCEDURE SBPlaySample(Len : Word; Cont : Boolean);

{ Mixer }
FUNCTION  MixerDetect : Boolean;

PROCEDURE MixerSetVolume(Reg : tMixerVolume; VolLeft, VolRight : Byte);
FUNCTION  MixerGetVolume(Reg : tMixerVolume; VAR VolLeft, VolRight : Byte) : Boolean;

{ SB Pro }
FUNCTION  SBProDetect : Boolean;
PROCEDURE SBProInit;
PROCEDURE SBProDone;

PROCEDURE SBProSetStereo(Stereo : Boolean);
PROCEDURE SBProSetFilter(Filter : Boolean);

{ SB 16 }
FUNCTION  SB16Detect : Boolean;
PROCEDURE SB16Init;
PROCEDURE SB16Done;

PROCEDURE SB16StartSample(Len : Word; Cont : Boolean);

IMPLEMENTATION

FUNCTION SBReset : Boolean;
CONST Ready = $AA;
VAR Ct, Stat : Byte;
BEGIN
  Port[DSPResetPort] := 1;
  For Ct := 1 to 100 do;
  Port[DSPResetPort] := 0;

  Stat := 0;
  Ct   := 0;
  WHILE (Stat <> Ready) AND (Ct < 100) DO
  BEGIN
    Stat := Port[DSPRStatPort];
    Stat := Port[DSPReadPort];
    Inc(Ct);
  END;

  SBReset := Stat = Ready;
END;

PROCEDURE SBWriteLoop(T : Word); ASSEMBLER;
ASM
  MOV     BX,t
  MOV     DX,[DSPWritePort]

@@lp:
  DEC     BX
  JZ      @@fin
  IN      AL,DX
  ADD     AL,AL
  JC      @@lp

@@fin:
  OR      BL,BH
  MOV     [SBWorksOk],BL
END;

PROCEDURE SBWriteByte(T : Word; B : Byte); ASSEMBLER;
ASM
  MOV     AL,b
  XOR     AH,AH
  PUSH    AX
  PUSH    $60
{  CALL    WriteSNum}

  MOV     AX,t
  PUSH    AX
  CALL    SBWriteLoop
  JNZ     @@ya

  MOV     DX,[DSPLifePort]
  IN      AL,DX

  MOV     AX,t
  PUSH    AX
  CALL    SBWriteLoop

@@ya:
  MOV     AL,b
  OUT     DX,AL

  MOV     AL,[SBWorksOk]
  ADD     AL,'A'
  XOR     AH,AH
  PUSH    AX
  PUSH    $40
{  CALL    WriteChar}
END;

PROCEDURE SBReadLoop(T : Word); ASSEMBLER;
ASM
  MOV     BX,t
  MOV     DX,[DSPRStatPort]

@@lp:
  DEC     BX
  JZ      @@fin
  IN      AL,DX
  ADD     AL,AL
  JNC     @@lp

@@fin:
  OR      BL,BH
  MOV     [SBWorksOk],BL
  MOV     DX,[DSPReadPort]
END;

FUNCTION SBReadByte(T : Word) : Byte; ASSEMBLER;
ASM
  MOV     AX,t
  PUSH    AX
  CALL    SBReadLoop
  JNZ     @@ya

@@ya:
  IN      AL,DX
END;

PROCEDURE SBWriteMixerReg(Reg, Val : Byte); ASSEMBLER;
ASM
  MOV     DX,[MixAddrPort]
  MOV     AL,[Reg]
  OUT     DX,AL

  MOV     DX,[MixDataPort]
  MOV     AL,[Val]
  OUT     DX,AL
END;

FUNCTION SBReadMixerReg(Reg : Byte) : Byte; ASSEMBLER;
ASM
  MOV     DX,[MixAddrPort]
  MOV     AL,[Reg]
  OUT     DX,AL

  MOV     DX,[MixDataPort]
  IN      AL,DX
END;

FUNCTION SBRegDetect : Boolean;
VAR Port, Lst : Word;
BEGIN
  SBRegDetect := SBRegDetected;
  If SBRegDetected then Exit;

  If SBPort < $8000 then
  BEGIN
    Port := SBPort;
    Lst  := SBPort;
  END else
  BEGIN
    Port := $210;
    Lst  := $280;
  END;

  WHILE (NOT SBRegDetected) AND (Port <= Lst) DO
  BEGIN
    CMS1DataPort := Port + CMS1DataPortOffset;
    CMS1AddrPort := Port + CMS1AddrPortOffset;
    CMS2DataPort := Port + CMS2DataPortOffset;
    CMS2AddrPort := Port + CMS2AddrPortOffset;

    MixAddrPort  := Port + MixAddrPortOffset;
    MixDataPort  := Port + MixDataPortOffset;

    FMStatPort   := Port + FMStatPortOffset;
    FMAddrPort   := Port + FMAddrPortOffset;
    FMDataPort   := Port + FMDataPortOffset;

    LFMStatPort  := Port + LFMStatPortOffset;
    LFMAddrPort  := Port + LFMAddrPortOffset;
    LFMDataPort  := Port + LFMDataPortOffset;

    RFMStatPort  := Port + RFMStatPortOffset;
    RFMAddrPort  := Port + RFMAddrPortOffset;
    RFMDataPort  := Port + RFMDataPortOffset;

    DSPResetPort := Port + DSPResetPortOffset;
    DSPReadPort  := Port + DSPReadPortOffset;
    DSPLifePort  := Port + DSPLifePortOffset;
    DSPWStatPort := Port + DSPWStatPortOffset;
    DSPWritePort := Port + DSPWritePortOffset;
    DSPRStatPort := Port + DSPRStatPortOffset;
    DSP8AckPort  := Port + DSP8AckPortOffset;
    DSP16AckPort := Port + DSP16AckPortOffset;

    CDDataPort   := Port + CDDataPortOffset;
    CDCmdPort    := Port + CDCmdPortOffset;
    CDStatPort   := Port + CDStatPortOffset;
    CDResetPort  := Port + CDResetPortOffset;
    CDEnablePort := Port + CDEnablePortOffset;

    SBRegDetected := SBReset;

    If NOT SBRegDetected then Inc(Port, $10);
  END;

  SBRegDetect := SBRegDetected;
END;

PROCEDURE SBRegInit;
BEGIN
  If NOT SBRegDetect then Exit;
  If NOT SBRegInited then
  BEGIN
    SBGetDSPVersion;
    DoHiSpeed := (SBVersion >= $200) AND SBHiSpeed;
    SBWriteByte(SBDefTimeout, sdcTurnOnSpeaker);
  END;
  SBRegInited := True;
END;

PROCEDURE SBRegDone;
BEGIN
  If NOT (SBRegDetected AND SBRegInited) then Exit;
  SBWriteByte(SBDefTimeout, sdcTurnOffSpeaker);
END;

PROCEDURE SBGetDSPVersion;
VAR I : Word;
    T : Word;
    S : String[2];
BEGIN
  SBWriteByte(SBDefTimeout, sdcGetDSPVersion);
  T := 0;
  REPEAT
    SBVersionMaj := SBReadByte($FFFF);
    Inc(T);
  UNTIL ((SBVersionMaj <> $AA) AND SBWorksOk) OR (T >= 10);
  SBVersionMin := SBReadByte(SBDefTimeout);

  Str(SBVersionMaj, SBVersionStr);
  SBVersionStr := SBVersionStr + '.';
  Str(SBVersionMin, S);
  If SBVersionMin > 9 then SBVersionStr := SBVersionStr +       S
                      else SBVersionStr := SBVersionStr + '0' + S;
END;

PROCEDURE SBGetCopyrightString;
VAR T : Word;
BEGIN
  SBWriteByte(SBDefTimeout, sdcGetCopyright);
  T := 0;
  REPEAT
    SBCopyright := Char(SBReadByte($FFFF));
    Inc(T);
  UNTIL ((SBCopyright[1] <> #$AA) AND SBWorksOk) OR (T = 10);

  WHILE SBWorksOk AND (Length(SBCopyright) < 80) DO
    SBCopyright := SBCopyright + Char(SBReadByte(SBDefTimeout));

  Dec(SBCopyright[0]);
END;

PROCEDURE SBSetTimeConst(Tc : Byte);
BEGIN
{  WriteChar('T', $04);}
  If SB16Detected then
  BEGIN
{    WriteChar('|', $04);}
    If SB16Bit
      then SBWriteByte(SBDefTimeout, $D9)
      else SBWriteByte(SBDefTimeout, $DA);
  END;
  SBWriteByte(SBDefTimeout,   sdcSetTimeConst); { Send time constant command.             }
  SBWriteByte(SBDefTimeout*4, Tc);              { Send the time constant.                 }
  TimeConst := 0;                               { Reset time constant to already changed. }
  If SB16Detected then If SB16Bit
    then SBWriteByte(SBDefTimeout, $47)
    else SBWriteByte(SBDefTimeout, $45);
END;

PROCEDURE SBUpdateTimeConst;
BEGIN
  If TimeConst = 0 then Exit;
  SBSetTimeConst(TimeConst);
END;

PROCEDURE SBStartSampleLS(Len : Word; Cont : Boolean);
BEGIN
  HSBlockSpecified := 0;

  SBWriteByte(SBDefTimeout, sdcStartLSpeedDMA);
  SBWriteByte(SBDefTimeout, Lo(Len));
  SBWriteByte(SBDefTimeout, Hi(Len));

{  WriteSNum(Len, $30);}
END;

PROCEDURE SBStartSampleHS(Len : Word; Cont : Boolean);
BEGIN
  If HSBlockSpecified <> Len then Cont := False;
  If NOT Cont then
  BEGIN
    SBWriteByte(SBDefTimeout, sdcSetHSpeedSize);
    SBWriteByte(SBDefTimeout, LO(Len));
    SBWriteByte(SBDefTimeout, HI(Len));
    HSBlockSpecified := Len;

{    WriteSNum(Len, $30);}
  END;

  If NOT (SB16Detected AND Cont) then SBWriteByte(SBDefTimeout, sdcStartHSpeedDMA);
END;

PROCEDURE SBPlaySample(Len : Word; Cont : Boolean);
BEGIN
  If Len < 10 then Exit;

  If SBStereo then Inc(Len, Len);
  Dec(Len);
{  WriteNum(60, Len, $70);}

  If SB16Detected AND (SBStereo OR SB16Bit)
    then SB16StartSample(Len, Cont)
    else If DoHiSpeed
      then SBStartSampleHS(Len, Cont)
      else SBStartSampleLS(Len, Cont);
END;

{----------------------------------------------------------------------------}
{ Mixer generic routines.                                                    }
{____________________________________________________________________________}

FUNCTION MixerDetect : Boolean;
VAR SaveReg : Word;
    NewReg  : Word;
BEGIN
  MixerDetect := MixerDetected;
  If NOT SBRegDetect OR MixerDetected then Exit;

  SaveReg := SBReadMixerReg($22);
  SBWriteMixerReg($22, 243);
  NewReg  := SBReadMixerReg($22);

  If NewReg = 243 then MixerDetected := True;
  SBWriteMixerReg($22, SaveReg);

  MixerDetect := MixerDetected;
END;

PROCEDURE MixerSetVolume(Reg : tMixerVolume; VolLeft, VolRight : Byte);
VAR Addr   : Byte;
    VolMax : Byte;
BEGIN
  If NOT MixerDetected then Exit;
  If SB16Detected
    then Addr := SB16Regs [Reg]
    else Addr := SBProRegs[Reg];

  If VolLeft > VolRight
    then VolMax := VolLeft
    else VolMax := VolRight;

  CASE Reg OF
    mvMicrophone : BEGIN
      If SB16Detected
        then SBWriteMixerReg(Addr, VolMax)
        else SBWriteMixerReg(Addr, VolMax SHR 5);
    END;
    mvSpeaker    : If SB16Detected then SBWriteMixerReg(Addr, VolMax);
    else If SB16Detected then
    BEGIN
      SBWriteMixerReg(Addr,     VolLeft);
      SBWriteMixerReg(Addr + 1, VolRight);
    END else SBWriteMixerReg(Addr, (VolLeft AND $F0) + (VolRight SHR 4));
  END;
END;

FUNCTION MixerGetVolume(Reg : tMixerVolume; VAR VolLeft, VolRight : Byte) : Boolean;
VAR Addr   : Byte;
    VolMax : Byte;
BEGIN
  MixerGetVolume := False;
  If NOT MixerDetected then Exit;

  If SB16Detected
    then Addr := SB16Regs[Reg]
    else Addr := SBProRegs[Reg];

  VolLeft  := 0;
  VolRight := 0;

  MixerGetVolume := True;

  CASE Reg OF
    mvMicrophone : BEGIN
      If SB16Detected
        then VolLeft := SBReadMixerReg(Addr)
        else VolLeft := SBReadMixerReg(Addr) SHL 5;
        VolRight := VolLeft;
    END;
    mvSpeaker : BEGIN
      If SB16Detected
        then VolLeft := SBReadMixerReg(Addr)
        else MixerGetVolume := False;
      VolRight := VolLeft;
    END;
    else If SB16Detected then
    BEGIN
      VolLeft  := SBReadMixerReg(Addr);
      VolRight := SBReadMixerReg(Addr + 1);
    END else
    BEGIN
      VolLeft  := SBReadMixerReg(Addr);
      VolRight := VolLeft SHL 4;
      VolLeft  := VolLeft AND $F0;
    END;
  END;
END;

{----------------------------------------------------------------------------}
{ Sound Blaster Pro generic routines.                                        }
{____________________________________________________________________________}

FUNCTION SBProDetect : Boolean;
BEGIN
  SBProDetect := SBProDetected;
  If SBProDetected then Exit;

  If NOT SBRegInited then SBRegInit;

  SBProDetected := SBRegDetect AND MixerDetect AND (SBVersion < $400);
  SBProDetect   := SBProDetected;
END;

PROCEDURE SBProInit;
BEGIN
  If NOT SBProDetect then Exit;
  SBProInited := True;
END;

PROCEDURE SBProDone;
BEGIN
  SBRegDone;
END;

PROCEDURE SBProSetStereo(Stereo : Boolean);
VAR I : Byte;
BEGIN
  If NOT SBProDetected then Exit;
  SBStereo := Stereo;
  I := SBReadMixerReg(mxrOutSetting);
  SBWriteMixerReg(mxrOutSetting, (I AND NOT mxoStereoOn) + (Byte(Stereo) * mxoStereoOn));
END;

PROCEDURE SBProSetFilter(Filter : Boolean);
VAR I : Byte;
BEGIN
  If NOT SBProDetected then Exit;
  SBFilter := Filter;
  I := SBReadMixerReg(mxrOutSetting);
  SBWriteMixerReg(mxrOutSetting, (I AND NOT mxoFilterNeg) + (Byte(Filter) * mxoFilterNeg));
END;

{----------------------------------------------------------------------------}
{ Sound Blaster 16 generic routines.                                         }
{____________________________________________________________________________}

FUNCTION SB16Detect : Boolean;
BEGIN
  SB16Detect := SB16Detected;
  If SB16Detected then Exit;
  If NOT SBRegInited then SBRegInit;

  SB16Detected := SBRegDetect AND MixerDetect AND (SBVersion >= $400);
  SB16Detect   := SB16Detected;
END;

PROCEDURE SB16Init;
BEGIN
  If NOT SB16Detect then Exit;
  SBGetCopyrightString;

  SB16Inited := True;
END;

PROCEDURE SB16Done;
BEGIN
  SBRegDone;
END;

PROCEDURE SB16StartSample(Len : Word; Cont : Boolean);
BEGIN
  If (NOT Cont) OR (SB16BlockSpecified <> Len) then
  BEGIN
    If SB16Bit
      then SBWriteByte(SBDefTimeout, $B6)
      else SBWriteByte(SBDefTimeout, $C6);
    If SBStereo
      then SBWriteByte(SBDefTimeout, $20)
      else SBWriteByte(SBDefTimeout, $00);
    SBWriteByte(SBDefTimeout, LO(Len));
    SBWriteByte(SBDefTimeout, HI(Len));
    SB16BlockSpecified := Len;
  END else
  BEGIN
    If SB16Bit
      then SBWriteByte(SBDefTimeout, $47)
      else SBWriteByte(SBDefTimeout, $45);
  END;
END;

END.
